// kiwiDlg.cpp : implementation file
//

#include "stdafx.h"
#include "kiwi.h"
#include "kiwiDlg.h"

//add in pdf.h into header files, if not add in head files, should do this
//#include <e:\kiwi\pdf.h>
#include "pdf.h"
//start of Byron add in
#include <fstream> //for read data
#include <vector>  //for store data in vector
#include <algorithm> //for sort data
#include <set>
#include <functional>

#include <string>
// writing on a text file
#include <iostream>

#include <stdio.h> //for time
#include <stdlib.h> //for time
#include <time.h> //for time
#include <math.h>

using namespace std;
/*
Type information in my home computer:
Number of bits in a character: 8
Size of character types is 1 byte
Signed char min: -128 max: 127
Unsigned char min: 0 max: 255
Default char is signed

Size of short int types is 2 bytes
Signed short min: -32768 max: 32767
Unsigned short min: 0 max: 65535

Size of int types is 4 bytes
Signed int min: -2147483648 max: 2147483647
Unsigned int min: 0 max: 4294967295

Size of long int types is 4 bytes
Signed long min: -2147483648 max: 2147483647
Unsigned long min: 0 max: 4294967295
*/

/* limits of this program "kiwi"
(1)number of columns is limited to 32767, big enough.
example: SSQUARE seqs; //storing all the sequences
	     SSQUARE indexes; //storing indexes of columns for sequences

(2)number of rows is limited to 65535, change UROW to LROW can increase the limit to 4294967295
example: USQUARE candidatesSupports (numOfCandidates); //containing supporting rows for the candidate of the same index in candidates, now used for first level only
         USQUARE patternsSupports;
         USQUARE kpatterns;
         USQUARE candidatesSupports (numOfCandidates);
         USQUARE s;
         USQUARE queryClusters2 (forSort.size());
         USQUARE queryClusters; // have the same size as queryPatterns. each row contains ids for rows supporting the pattern in queryPatterns with the same index
         USQUARE chosenClusters; //chosen for writing or show
		 USET dif;
possible change: change US**ARE to LS**ARE, US*T to LS*T

(3)k is limited to 4294967295, big enough
example: LCUBE allPatterns; //storing patterns for all levels, it will store indexes for patterns too, so cannot go beyond 65535
		 allPatterns = UCUBE(levels, s);
possible change: change UCUBE to LCUBE

(4)k * d is limited to 4294967295, where d is # of columns of the input data matrix, i.e., dimensionality. almost no need to change. If needed, use unsigned long long (undefined here).
example: ULPAIR s; 
         vector < ULP**R > patterns; //store index (for candidates, can be used to derive pointer to previous item and current item index) and support counts

(5)queryPatterns and queryClusters are limited to 4294967295, big enough.
example: LSQUARE forSort(queryClusters.size());
-----------------------
kiwi2, allows number of rows to go beyond 65535 (bounded by 4294967295)

The needed changes are commented in the codes. search for scalalbe2, and make the recommended change

Note: also need to change the (for int ...) to (for unsigned int ...), I didn't do it, so,
actually upper bounded by 2147483647
-------------------------
*/

typedef vector<short> SROW; //-32767 to 32767, can be used to dimension ids, enough, so # of columns is limited to 32767
typedef vector<SROW> SSQUARE;

typedef vector<unsigned short> UROW; //0 - 65535, used for rows and indexes, so # of rows is limited to 65535
typedef vector<UROW> USQUARE;  //for candidatesSupports and patternsSupports
typedef vector<USQUARE> UCUBE;

typedef vector<unsigned long> LROW; //0 - 4294967295. using 4 bypes, same as int
typedef vector<LROW> LSQUARE; //not used in this version "kiwi.exe"
typedef vector<LSQUARE> LCUBE;//not used in this version "kiwi.exe"

typedef pair <unsigned short, unsigned long> ULPAIR;
typedef pair <unsigned long, unsigned long> LLPAIR; //used for scalalbe2, since for datasets with > 65355 rows, the support may be bigger than than 65535 too.

typedef set<unsigned short> USET; 
typedef set<unsigned long> LSET; //not used in this version "kiwi.exe"

//end of Byron add in

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CKiwiDlg dialog

CKiwiDlg::CKiwiDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CKiwiDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CKiwiDlg)
	m_cols = 0;
	m_dataSource = _T("breastcancer.txt");
	m_rows = 0;
	m_writeToDir = _T("");
	m_readDataTime = 0.0f;
	m_show = _T("");
	m_k = 10;
	m_w = 10;
	m_clusteringTime = 0.0f;
	m_min_row = 2;
	m_noise = -999;
	m_seqLength = 0;
	m_queryMinCol = 0;
	m_queryTopK = 0;
	m_queryLevelHigh = 2;
	m_queryLevelLow = 2;
	m_queryMinSupport = 2;
	m_queryRowIndexHigh = 0;
	m_sizeOfQueryPatterns = 0;
	m_queryRowIndexLow = 0;
	m_queryLevel = -1;
	m_queryRowIndex = -1;
	m_scanTime = 0.0f;
	m_actual = true;
	m_patternSource = _T(".pat");
	m_closed = false;
	m_removeMarked = true;
	m_numberOfClusters = 0;
	m_timestampInFileName = true;
	m_queryTime = 0.0f;
	m_memory = -1; // -1 means do not set up memory limitation
	m_subset = 0;
	m_kplus = 0;
	m_deficit = 0;
	m_levels = 0;
	m_support = 0;
	m_probability = 0.0;
	m_seq1 = 0;
	m_seq2 = 0;
	m_longestLength = 0;
	m_LCStime = 0.0f;
	m_autoTime = 0.0f;
	m_extend = true;
	m_dimensionalityHigh = 0;
	m_dimensionalityLow = 0;
	m_clusterSizeHigh = 0;
	m_clusterSizeLow = 0;
	m_numberOfMarked = 0;
	m_useLCS = true;
	m_clusterSource = _T(".clu");
	m_howToShowPatterns = FALSE;
	m_markRepeats = true;
	m_seq = 0;
	m_seqLengthThreshold = 2;
	m_numberOfRemoved = 0;

	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CKiwiDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CKiwiDlg)
	DDX_Control(pDX, IDC_SPINsubset, m_spinSubset);
	DDX_Text(pDX, IDC_cols, m_cols);
	DDX_Text(pDX, IDC_DataSource, m_dataSource);
	DDX_Text(pDX, IDC_rows, m_rows);
	DDX_Text(pDX, IDC_WriteToDir, m_writeToDir);
	DDX_Text(pDX, IDC_readDataTime, m_readDataTime);
	DDX_Text(pDX, IDC_show, m_show);
	DDV_MaxChars(pDX, m_show, 50000000);
	DDX_Text(pDX, IDC_k, m_k);
	DDX_Text(pDX, IDC_w, m_w);
	DDX_Text(pDX, IDC_clusteringTime, m_clusteringTime);
	DDX_Text(pDX, IDC_min_row, m_min_row);
	DDX_Text(pDX, IDC_Noise, m_noise);
	DDX_Text(pDX, IDC_seqLength, m_seqLength);
	DDX_Text(pDX, IDC_QueryTopK, m_queryTopK);
	DDX_Text(pDX, IDC_QueryLevelHigh, m_queryLevelHigh);
	DDX_Text(pDX, IDC_QueryLeveLow, m_queryLevelLow);
	DDX_Text(pDX, IDC_QueryMinSupport, m_queryMinSupport);
	DDX_Text(pDX, IDC_QueryRowIndexHigh, m_queryRowIndexHigh);
	DDX_Text(pDX, IDC_sizeOfQueryPatterns, m_sizeOfQueryPatterns);
	DDX_Text(pDX, IDC_QueryRowIndexLow, m_queryRowIndexLow);
	DDX_Text(pDX, IDC_scanTime, m_scanTime);
	DDX_Check(pDX, IDC_actual, m_actual);
	DDX_Text(pDX, IDC_PatternSource, m_patternSource);
	DDX_Check(pDX, IDC_closed, m_closed);
	DDX_Check(pDX, IDC_removeMarked, m_removeMarked);
	DDX_Text(pDX, IDC_numberOfClusters, m_numberOfClusters);
	DDX_Check(pDX, IDC_timestampInFileName, m_timestampInFileName);
	DDX_Text(pDX, IDC_queryTime, m_queryTime);
	DDX_Text(pDX, IDC_memory, m_memory);
	DDX_Text(pDX, IDC_subset, m_subset);
	DDV_MinMaxInt(pDX, m_subset, 0, 5);
	DDX_Text(pDX, IDC_kplus, m_kplus);
	DDX_Text(pDX, IDC_deficit, m_deficit);
	DDX_Text(pDX, IDC_levels, m_levels);
	DDX_Text(pDX, IDC_support, m_support);
	DDX_Text(pDX, IDC_probability, m_probability);
	DDX_Text(pDX, IDC_seq1, m_seq1);
	DDX_Text(pDX, IDC_seq2, m_seq2);
	DDX_Text(pDX, IDC_longestLength, m_longestLength);
	DDX_Text(pDX, IDC_LCStime, m_LCStime);
	DDX_Text(pDX, IDC_autoTime, m_autoTime);
	DDX_Check(pDX, IDC_extend, m_extend);
	DDX_Text(pDX, IDC_dimensionalityHigh, m_dimensionalityHigh);
	DDX_Text(pDX, IDC_dimensionalityLow, m_dimensionalityLow);
	DDX_Text(pDX, IDC_clusterSizeHigh, m_clusterSizeHigh);
	DDX_Text(pDX, IDC_clusterSizeLow, m_clusterSizeLow);
	DDX_Text(pDX, IDC_numberOfMarked, m_numberOfMarked);
	DDX_Check(pDX, IDC_useLCS, m_useLCS);
	DDX_Text(pDX, IDC_ClusterSource, m_clusterSource);
	DDX_Check(pDX, IDC_HowToShowPatterns, m_howToShowPatterns);
	DDX_Check(pDX, IDC_markRepeats, m_markRepeats);
	DDX_Text(pDX, IDC_seq, m_seq);
	DDX_Text(pDX, IDC_seqLengthThreshold, m_seqLengthThreshold);
	DDX_Text(pDX, IDC_numberOfRemoved, m_numberOfRemoved);

	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CKiwiDlg, CDialog)
	//{{AFX_MSG_MAP(CKiwiDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_ReadData, OnReadData)
	ON_BN_CLICKED(IDC_Clear, OnClear)
	ON_BN_CLICKED(IDC_ShowSeqs, OnShowSeqs)
	ON_BN_CLICKED(IDC_ShowColumns, OnShowColumns)
	ON_BN_CLICKED(IDC_ShowRows, OnShowRows)
	ON_BN_CLICKED(IDC_ShowIndexes, OnShowIndexes)
	ON_BN_CLICKED(IDC_Clustering, OnClustering)
	ON_BN_CLICKED(IDC_Preprocessing, OnPreprocessing)
	ON_BN_CLICKED(IDC_ReadPattern, OnReadPattern)
	ON_BN_CLICKED(IDC_QueryGetPatterns, OnQueryGetPatterns)
	ON_BN_CLICKED(IDC_QueryShowPatterns, OnQueryShowPatterns)
	ON_BN_CLICKED(IDC_QueryGetClusters, OnQueryGetClusters)
	ON_BN_CLICKED(IDC_ShowClusters, OnShowClusters)
	ON_BN_CLICKED(IDC_WriteClusters, OnWriteClusters)
	ON_BN_CLICKED(IDC_Help, OnHelp)
	ON_BN_CLICKED(IDC_calculateProbability, OncalculateProbability)
	ON_BN_CLICKED(IDC_doubling, Ondoubling)
	ON_BN_CLICKED(IDC_LCS, OnLCS)
	ON_BN_CLICKED(IDC_LCSAll, OnLCSAll)
	ON_BN_CLICKED(IDC_Auto, OnAuto)
	ON_BN_CLICKED(IDC_ChooseClusters, OnChooseClusters)
	ON_BN_CLICKED(IDC_ReadClusters, OnReadClusters)
	ON_BN_CLICKED(IDC_ShrinkClu, OnShrinkClu)
	ON_BN_CLICKED(IDC_Clustering2, OnClustering2)
	ON_BN_CLICKED(IDC_ShowAllPatterns, OnShowAllPatterns)
	ON_BN_CLICKED(IDC_about, Onabout)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKiwiDlg message handlers

BOOL CKiwiDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	m_spinSubset.SetRange(0, 5);
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CKiwiDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CKiwiDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CKiwiDlg::writeFile(CString FileName, CString toWrite)
{
	CFile f(FileName,CFile::modeCreate | CFile::modeWrite);
	//f.Write((LPCTSTR) toWrite, toWrite.GetLength());
	//work the same
	f.Write(toWrite, toWrite.GetLength());
	f.Close(); //without this, it works the same
}

//read data file and preprocessing each row into a seq, 
//i.e., sort columns according to values
//also, prepare indexes, from which, given seq #, given column, return index in the seq.
void CKiwiDlg::OnReadData() 
{
	UpdateData(true);
	//initialization
	m_rows = 0;
	m_cols = 0;
	columns.clear();
	rows.clear();
	seqs.clear();
	indexes.clear();
	mark.clear();
	m_show = "";
	

	//start reading data
	m_readDataTime = 0;
	clock_t start, finish;
	start = clock();

	if(m_dataSource == "")
	{
		m_show = "Open File Error !";
		UpdateData(false);
		return;
	}
	
	ifstream inFile;
	inFile.open(m_dataSource);
	if(!inFile.is_open())
	{
		m_show = "Open File Error !";
		UpdateData(false);
		return;
	}

	m_numberOfRemoved = 0;
	const signed long MAX_LINE_SIZE = 100000; // suppose MAX_LINE_SIZE = 100000;
	char line[ MAX_LINE_SIZE ]; 
	char * token;

	//read in the first line, which contains the columns
	inFile.getline(line, MAX_LINE_SIZE);
	token = strtok(line, "\t"); //reading TAB as the delimiter
	token = strtok(NULL, "\t"); //get the next token (and forget the first one, which is columns, or gid something)
	while( token )
	{
		columns.push_back(token);
		token = strtok(NULL, "\t" );
	}
	
	while(!inFile.eof())
	{
		inFile.getline(line, MAX_LINE_SIZE );
		token = strtok(line, "\t" ); //the first token contains info about gid
		if(token == NULL) //added this, so, skip any empty lines
			continue;
		rows.push_back (token);
		token = strtok(NULL, "\t"); //get the next token
		
		short index = 0; //the column in the pair	
		vector < pair<double, short> > ps; // ps is a vector of pairs, each pair = <value, column>
		while(token )
		{
			ps.push_back(make_pair(atof(token), index));
			token = strtok(NULL, "\t" );
			index ++;
		}
		
		//preparing seqs;
		sort(ps.begin(), ps.end());
		SROW aSeq;

		vector < pair<double, short> >::iterator in;
		pair<double, short> p;

		for(in = ps.begin(); in!=ps.end(); in++)
		{	
			p = *in;
			if (p.first != m_noise) //taking care of noises, in original file, chance N/A (i.e., tab) to any number that is smallest, say, -999, over the whole dataset
				aSeq.push_back(p.second);
		}
		
		//remove seq with length < threshold
		if(aSeq.size() < m_seqLengthThreshold)
		{
			rows.pop_back (); //since push_back already;
			m_numberOfRemoved ++;
			continue;
		}
		
		seqs.push_back(aSeq);
		//preparing indexes
		
		//no matter with or without noises, use this to build indexes
		
		SROW anIndex (columns.size(), -1); // -1 means that item does not exist
		for (int e = 0; e < aSeq.size(); e++)
			anIndex[aSeq[e]] = e;
		indexes.push_back (anIndex);
	}

	inFile.close();
	finish = clock();
	m_readDataTime = (double)(finish - start) / CLOCKS_PER_SEC;
	m_rows = rows.size();
	m_cols = columns.size();

	//open source. display number of removed in the display window instead of the edit box
	if(m_numberOfRemoved != 0)
	{
		m_show.Format(_T("%d"), m_numberOfRemoved);
		m_show = "number of removed rows = " + m_show;
	}
	

	UpdateData(false);
}

//use tab as the delimiter
CString CKiwiDlg::toStringROW(SROW v)
{
		if(v.empty())
		return "";

	CString str, lineStr;
		
	for(int i = 0; i < v.size(); i++)
	{	
		str.Format(_T("%d"), v[i]);
		lineStr = lineStr + str + "\t";
	}
	lineStr.Delete(lineStr.GetLength()-1, 1);
		return lineStr;
}

//use "," as the delimiter
CString CKiwiDlg::toStringROW2(SROW v)
{
	if(v.empty())
		return "";

	CString str, lineStr;
		
	for(int i = 0; i < v.size(); i++)
	{	
		str.Format(_T("%d"), v[i]);
		lineStr = lineStr + str + ",";
	}
	lineStr.Delete(lineStr.GetLength()-1, 1);
	return lineStr;
}

//used "," as the delimiter
CString CKiwiDlg::toStringROW2(UROW v)
{
	if(v.empty())
		return "";

	CString str, lineStr;
		
	for(int i = 0; i < v.size(); i++)
	{	
		str.Format(_T("%d"), v[i]);
		lineStr = lineStr + str + ",";
	}
	lineStr.Delete(lineStr.GetLength()-1, 1);
		return lineStr;
}

//use "," as the delimiter
CString CKiwiDlg::toStringROW2(LROW v)
{
	if(v.empty())
		return "";

	CString str, lineStr;
		
	for(int i = 0; i < v.size(); i++)
	{	
		str.Format(_T("%d"), v[i]);
		lineStr = lineStr + str + ",";
	}
	lineStr.Delete(lineStr.GetLength()-1, 1);
		return lineStr;
}

CString CKiwiDlg::toStringSQUARE(SSQUARE m)
{

	CString output;
	for(int i = 0; i < m.size(); i++)
		output += toStringROW(m[i]) + "\r\n";
	
	return output;
}

CString CKiwiDlg::toStringSQUARE(USQUARE m)
{

	CString output;
	for(int i = 0; i < m.size(); i++)
		output += toStringROW(m[i]) + "\r\n";
	
	return output;
}

CString CKiwiDlg::toStringSQUARE(LSQUARE m)
{

	CString output;
	for(int i = 0; i < m.size(); i++)
		output += toStringROW(m[i]) + "\r\n";
	
	return output;
}

//use tab as the delimiter
CString CKiwiDlg::toStringROW(UROW v)
{
	if(v.empty())
		return "";

	CString str, lineStr;
		
	for(int i = 0; i < v.size(); i++)
	{	
		str.Format(_T("%d"), v[i]);
		lineStr = lineStr + str + "\t";
	}
	lineStr.Delete(lineStr.GetLength()-1, 1);
		return lineStr;
}

//use tab as the delimiter
CString CKiwiDlg::toStringROW(LROW v)
{
	if(v.empty())
		return "";

	CString str, lineStr;
		
	for(int i = 0; i < v.size(); i++)
	{	
		str.Format(_T("%d"), v[i]);
		lineStr = lineStr + str + "\t";
	}
	lineStr.Delete(lineStr.GetLength()-1, 1);
		return lineStr;
}



void CKiwiDlg::OnShowColumns() 
{
	m_show = "";
	CString str, lineStr;
	
	vector<CString>::iterator in;
	vector<CString> temp;
	int index = 0;
	for(in = columns.begin(); in!=columns.end(); in++)
	{	
		str.Format(_T("%d"), index);
		lineStr = str + "\t" + *in;
		m_show = m_show + lineStr + "\r\n";
		index ++;
	}
	UpdateData(false);	
}

void CKiwiDlg::OnShowRows() 
{
	m_show = "";
	CString str, lineStr;
	
	vector<CString>::iterator in;
	vector<CString> temp;
	int index = 0;
	for(in = rows.begin(); in!=rows.end(); in++)
	{	
		str.Format(_T("%d"), index);
		lineStr = str + "\t" + *in;
		m_show = m_show + lineStr + "\r\n";
		index ++;
	}
	UpdateData(false);	
	
}

//if not in the range, for all seqs
void CKiwiDlg::OnShowSeqs() 
{
	UpdateData(true);
	if(m_seq >= 0 && m_seq < m_rows)
	{
		CString str;
		m_seqLength = seqs[m_seq].size();
		str.Format(_T("%d"), m_seqLength);
		m_show = "The length of the sequence = " + str + "\r\n";
		m_show += toStringROW(seqs[m_seq]) + "\r\n";
		UpdateData(false);
	}
	else if (m_seq < 0)
	{
		m_show = toStringSQUARE(seqs);
		m_seqLength = m_cols;
	}
	else
		m_show += "\r\nOut of range! Enter -1 to show all!\r\n";
		
	UpdateData(false);
}

void CKiwiDlg::OnShowIndexes() 
{
	UpdateData(true);
	if(m_seq >= 0 && m_seq < m_rows)
	{
		m_show += toStringROW(indexes[m_seq]) + "\r\n";
		m_seqLength = seqs[m_seq].size();
		UpdateData(false);
	}
	else if (m_seq < 0)
	{
		m_show = toStringSQUARE(indexes);
		m_seqLength = m_cols;
	}
	else
		m_show += "\r\nOut of range! Enter -1 to show all!\r\n";
		
	UpdateData(false);
}

//for clustering
bool rankSupport (ULPAIR elem1, ULPAIR elem2)//kiwi2, change to LLPAIR
{
	return (elem1.first > elem2.first); //since position 0 stores support count
}

//for clustering2, since in clustering2, patterns are a vector of LLPAIR
bool rankSupport2 (LLPAIR elem1, LLPAIR elem2)//kiwi2, change to LLPAIR
{
	return (elem1.first > elem2.first); //since position 0 stores support count
}

void CKiwiDlg::OnClustering() 
{
	UpdateData(true);
	if(seqs.empty())
	{
		m_show = "Read File First !";
		UpdateData(false);
		return;
	}
	
	//initialization
	allPatterns.clear();
	queries.clear();
	queryPatterns.clear();
	queryClusters.clear();
	chosenPatterns.clear();
	chosenClusters.clear();
	mark.clear();
	m_sizeOfQueryPatterns = 0;
	m_deficit = 0;

	if(m_w > columns.size())
		m_w = columns.size();
	
	int columnSize = columns.size();
	
	//output will be used for output
	CString output = "";
	//collect basic information
	char charStr [20];
	CString str;	
	output += "File Name = " + m_dataSource + "\r\n";
	//date and time info
	_strdate( charStr);
    str = charStr;
	str = "Date = " + str + "\r\nTime = ";
	output += str;
	_strtime( charStr);
	str =  charStr;
	output += str + "\r\n\r\n";
	
	str.Format(_T("%d"), m_rows);
	output += "number of rows = " + str + "\r\n";
	str.Format(_T("%d"), m_cols);
	output += "number of columns = " + str + "\r\n\r\n";
	
	//for parameter settings
	output += "Parameter setting:\r\n";
	str.Format(_T("%d"), m_noise);
	output += "noise = " + str + "\r\n";
	str.Format(_T("%d"), m_seqLengthThreshold);
	output += "length >= " + str + "\r\n";
	str.Format(_T("%d"), m_numberOfRemoved);
	output += "removed = " + str + "\r\n";
	
	str.Format(_T("%d"), m_k);
	output += "k = " + str + "\r\n";
	str.Format(_T("%d"), m_w);
	output += "w = " + str + "\r\n";
	str.Format(_T("%d"), m_min_row);
	output += "min_row = " + str + "\r\n";
	str.Format(_T("%d"), m_memory);
	output += "memory = " + str + "\r\n";
	str.Format(_T("%d"), m_subset);
	output += "remove redundancy = " + str + "\r\n";
	str.Format(_T("%d"), m_kplus);
	output += "kplus = " + str + "\r\n";
	str.Format(_T("%d"), m_deficit);
	output += "deficit = " + str + "\r\n\r\n";
		
	//Program running information
	output += "Program running information:\r\n";
		
	//start running the first level
	clock_t start, finish, levelStart, levelFinish;
	start = clock();
	levelStart = clock();
	
	//for 1-item
	int numOfCandidates = columnSize;
	ULPAIR s; //so, k * d is upper bounded by 4294967295//kiwi2, change to LLPAIR
	// each pair initially contians count = 0 and the index of that pair
	//the index can be used to caculate the corresponding item. For the first level, the size of candidates is d
	//and the indexes are column (item) ids themselves.
	//the other levels, the size of candidates is k x d. For each index i, i div d returns a pointer to the pattern in 
	//previous level, and i mod d returns the column (item) id for the current level.
	//and will be attatched with seq id's that support the candidate
	USQUARE candidatesSupports (numOfCandidates); //having the same size as candidates, containing supporting rows for the candidate of the same index in candidates	//kiwi2, change to LSQUARE
	//candidatesSupports is now used only for the first level. On other levels, count supports first, then choose the top k ones, and then get their supports.
	//saves lots of memory, a lot faster than before
	//now, only used for the first level
	vector < ULPAIR > patterns; //support count and store index (for candidates, can be used to derive pointer to previous item and current item index) //kiwi2, change to LLPAIR
	USQUARE patternsSupports;//kiwi2, change to LSQUARE
	
	//insert seqs of width m_w into candidates and supports to count support
	for (int j = 0; j < seqs.size(); j ++)
		for(int w = 0; w < m_w; w++)
		{
			if(w < seqs[j].size()) //doesn't need this if no noises
			{
				int v = seqs[j][w];
				candidatesSupports[v].push_back(j);
			}
		}
	
	//insert information into patterns
	for (int i = 0; i < numOfCandidates; i++)
	{
		if(candidatesSupports[i].size() >= m_min_row)
			patterns.push_back (make_pair(candidatesSupports[i].size(), i));
	}

	//get realSize
	int realSize = min(m_k, patterns.size());
	
	//partial sort candidates and use the top k indexes to resize supports
	partial_sort(patterns.begin(), patterns.begin() + realSize, patterns.end(), rankSupport);

	//now, adjust realSize according to memory limitation
	//allowedTotalSupport shoudl be < m_memory (MB) * 1048576 / 2 (each unsigned int takes 2 bytes).
	unsigned long allowedTotalSupport = m_memory * 1048576.0 / 2; //need to put it here since the loops may need it //kiwi2, change to 4
	if(m_memory != -1)
	{
		int sup = 0;
		for (int k = 0; k < realSize; k++)
		{
			sup += patterns[k].first;
			if(sup > allowedTotalSupport)
				break;
		}
		if (k + 1 < realSize)
			realSize = k;
	}

	//first round, do not need to remove subsets, since the possibility is low and candidates are not many
	
	patterns.resize(realSize);
	
	//patternSupport
	output += "Level 0:\r\n";
	int patternSupport = 0;
	for (int y = 0; y < patterns.size(); y++)
		patternSupport += patterns[y].first;
	str.Format(_T("%d"), patternSupport);
	output += "Pattern support = " + str + "\r\n";

	//insert only supporting rows for top k patterns into patternsSupports
	for (int c = 0; c < patterns.size(); c++)
		patternsSupports.push_back(candidatesSupports[patterns[c].second]);
	
	candidatesSupports.clear();
	//now, only patterns and patternsSupport left, they'd be useful for the next level
	
	//insert the level 1 patterns into allPatterns
	LSQUARE kpatterns;
	
	if(patterns.empty())
		return;
	
	for (int e = 0; e < patterns.size(); e++)
	{
		LROW s;
		s.push_back (e);
		s.push_back (0); //level 0, no previous level to point to
		s.push_back (patterns[e].second);  // second is the item
		s.push_back (patterns[e].first); // first is the support
		kpatterns.push_back(s);
	}
	
	allPatterns.push_back (kpatterns);
	kpatterns.clear();
		
	//get running time information for output
	double runTime;
	levelFinish = clock();
	runTime = (double)(levelFinish - levelStart) / CLOCKS_PER_SEC;
	str = gcvt(runTime, 5, charStr);
	output += "Run time = " + str + "\r\n\r\n";

	//for 2-item and more
	//patterns will be used for each level, the first column is its support (# of counts)
	//the second column is its index
		
	//Then the top k of them (if k > patterns.size()) will be chosen as the patterns of this level and stored into allPatterns
	//Then get the supporting rows for each pattern in patterns
	//allPatterns is a global variable, a vector of a vector of pairs
	//For each level, a vector of pairs is used to store the k patterns, a pair is <index for the item of previous level, item for current level>
		
	int level = 0; //for output information, curret level is 1
	while (true) //until support of the most frequent pattern < min_row;
	{
		levelStart = clock();
		level ++;
		numOfCandidates = allPatterns.back().size() * columnSize;
		
		vector < ULPAIR > patterns (numOfCandidates); //stores support count and index for each candidate//kiwi2, change to LLPAIR
		for (int z = 0; z < patterns.size(); z++)
			patterns[z] = make_pair(0, z);

		//insert relevant data into candidates to count support
		int seqNo;
		int startPos, endPos, candidateIndex;
		for (int f = 0; f < allPatterns.back().size(); f ++) //for each pattern
		{
			for(int p = 0; p < patternsSupports[f].size(); p++)  //for each supporting sequence
			{
				//get the position for the last item of the pattern in the sequence
				//since each column appears exactly once (if noise, at most once) in each sequence
				//short lastItem = patterns[f][1];
				//short posForLastItem = indexes[seqNo][lastItem];
				seqNo = patternsSupports[f][p];
				startPos = indexes[seqNo][allPatterns.back()[f][2]] + 1;
				endPos = min(startPos + m_w, seqs[seqNo].size());
				while(startPos < endPos) //for each pos after posForLastItem
				{
					candidateIndex = f * columnSize + seqs[seqNo][startPos]; //get index in candidates
					patterns[candidateIndex].first ++; //this is a lot faster than before. before, I push_back the gene id to candidateSupports...
					startPos++;
				}
			}
		}

	
		//eliminate patterns < min_row
		vector < ULPAIR > okpatterns; //kiwi2, change to LLPAIR
		for (int ok = 0; ok < patterns.size(); ok++)
		{
			if(patterns[ok].first >= m_min_row)
				okpatterns.push_back(patterns[ok]);
		}
		patterns = okpatterns;
		okpatterns.clear();

		//get realSize
		if(m_subset) //level >= 1 already
			realSize = min(m_k + m_kplus, patterns.size());
		else
			realSize = min(m_k, patterns.size());
		
		//partial sort patterns
		partial_sort(patterns.begin(), patterns.begin() + realSize, patterns.end(), rankSupport);
	
		//now, adjust realSize according to memory limitation
		if(m_memory != -1)
		{
			int sup = 0;
			for (int kk = 0; kk < realSize; kk++)
			{
				sup += patterns[kk].first;
				if(sup > allowedTotalSupport)
					break;
			}
			if (kk + 1 < realSize)
				realSize = kk;
		}
		
		patterns.resize(realSize);

		/*
		//remove patterns with support < min_row //this methods works only after sorting patterns
		int x;
		for (x = patterns.size() - 1; x >=0; x--)
		{
			if(patterns[x].first >= m_min_row)
				break;
		}
		patterns.resize(x + 1);
		*/
		/*while (! patterns.empty())
		{
			if(patterns.back().first < m_min_row)
				patterns.pop_back ();
			else
				break;
		}*/

		//insert only supporting rows for top k patterns into patternsSupports
		USQUARE temppatternsSupports(patterns.size());//kiwi2, change to LSQUARE
		for (int c = 0; c < patterns.size(); c++)
		{
			int pointer = patterns[c].second / columnSize;
			short item = patterns[c].second % columnSize;
			short lastItem = allPatterns.back()[pointer][2];
			for (int row = 0; row < patternsSupports[pointer].size(); row++)
			{
				//seqNo = patternsSupports[f][p];
				//startPos = indexes[seqNo][patterns[f].second] + 1;
				int seqNo = patternsSupports[pointer][row];
				int dif = indexes[seqNo][item] - indexes[seqNo][lastItem];
				if (dif > 0 && dif <= m_w)
					temppatternsSupports[c].push_back (seqNo);
			}
		}
		patternsSupports = temppatternsSupports;
		temppatternsSupports.clear();
				
		//now, remove subsets
		if(m_subset)//m_subset = 1, 2 or 3. 1 for check only the last item, 2 check back 2, 3 check all
		//the task is to try to get k patterns with no subsets until patterns gets empty
		//now, m_subset can be 4 or 5 too. 4 for equality of two supporting sets no matter items match or not
		//5 is for subset relationship of two supporting sets no matter items match or not
		{
			switch(m_subset)
			{
				case 1: //check item2 with item1, possibly remove pattern2 if it item2 == item1 and corresponding supporting rows of pattern2 is a subset of pattern1
				{
					for(int n2 = 1; n2 < patterns.size(); n2++) //starting with the 2nd element
					{
						if(n2 >= m_k) //reached m_k
							break;
						short item2 = patterns[n2].second % columnSize;
						USET sn2(patternsSupports[n2].begin(), patternsSupports[n2].end()); //kiwi2, change to LSET
						for(int n1 = n2 - 1; n1 >= 0; n1 --)
						{
							short item1 = patterns[n1].second % columnSize;
							if(item2 == item1)
							{
								if(patterns[n2].first == patterns[n1].first) //if same support, they must equal, otherwise, not subset of each other
									//in this case, since the last item of the two patterns are equal, remove either pattern would be ok
								{
									if(patternsSupports[n2] == patternsSupports[n1])
									{
										patternsSupports.erase(patternsSupports.begin() + n2);
										patterns.erase(patterns.begin() + n2);
										n2--; //the new one taking the place need to be checked too
										break;
									}
								}
								else
								{
									USET sn1(patternsSupports[n1].begin(), patternsSupports[n1].end());//kiwi2, change to LSET
									USET dif;//kiwi2, change to LSET
									set_difference(sn2.begin(),sn2.end(),sn1.begin(),sn1.end(),insert_iterator<USET>(dif,dif.begin()));//kiwi2, change to LSET
									if(dif.empty()) 
									{
										patternsSupports.erase(patternsSupports.begin() + n2);
										patterns.erase(patterns.begin() + n2);
										n2--;
										break;
									}
								}
							}
						}
					}
				}
				break;
				case 2: //check item2 with item1 and lastItem1
				{
					for(int n2 = 1; n2 < patterns.size(); n2++) //starting with the 2nd element
					{
						if(n2 >= m_k) //reached m_k
							break;
						short item2 = patterns[n2].second % columnSize;
						int ptLastItem2 = patterns[n2].second / columnSize;
						short lastItem2 = allPatterns[level - 1][ptLastItem2][2];

						USET sn2(patternsSupports[n2].begin(), patternsSupports[n2].end()); //kiwi2, change to LSET
						for(int n1 = n2 - 1; n1 >= 0; n1 --)
						{
							short item1 = patterns[n1].second % columnSize;
							if(item2 == item1 || lastItem2 == item1)
							{
								if(patterns[n2].first == patterns[n1].first) //if same support, they must equal, otherwise, not subset of each other
								//in this case, pattern1 has bettern potential to grow longer, so kill pattern2
								{
									if(patternsSupports[n2] == patternsSupports[n1])
									{
										patternsSupports.erase(patternsSupports.begin() + n2);
										patterns.erase(patterns.begin() + n2);
										n2--; //the new one taking the place need to be checked too
										break;
									}
								}
								else
								{
									USET sn1(patternsSupports[n1].begin(), patternsSupports[n1].end());//kiwi2, change to LSET
									USET dif;//kiwi2, change to LSET
									set_difference(sn2.begin(),sn2.end(),sn1.begin(),sn1.end(),insert_iterator<USET >(dif,dif.begin()));//kiwi2, change to LSET
									if(dif.empty()) 
									{
										patternsSupports.erase(patternsSupports.begin() + n2);
										patterns.erase(patterns.begin() + n2);
										n2--;
										break;
									}
								}
							}
						}
					}
				}
				break;
				case 3: //check item2 with all items in pattern1
				{
					for(int n2 = 1; n2 < patterns.size(); n2++) //starting with the 2nd element
					{
						if(n2 >= m_k) //reached m_k
							break;
						vector <int> items2;
						items2.push_back (patterns[n2].second % columnSize);
						int ptItem2 = patterns[n2].second / columnSize;
						for (int le = level - 1; le >= 0; le --)
						{
							items2.push_back (allPatterns[le][ptItem2][2]);
							ptItem2 = allPatterns[le][ptItem2][1];
						}
						USET sn2(patternsSupports[n2].begin(), patternsSupports[n2].end()); //kiwi2, change to LSET
						for(int n1 = n2 - 1; n1 >= 0; n1 --)
						{
							short item1 = patterns[n1].second % columnSize;
							//if find items2 in items1
							if(find(items2.begin(), items2.end(), item1) != items2.end())
							{
								if(patterns[n2].first == patterns[n1].first) //if same support, they must equal, otherwise, not subset of each other
								//in this case, pattern1 has potential to grow longer, so kill pattern2
								{
									if(patternsSupports[n2] == patternsSupports[n1])
									{
										patternsSupports.erase(patternsSupports.begin() + n2);
										patterns.erase(patterns.begin() + n2);
										n2--; //the new one taking the place need to be checked too
										break;
									}
								}
								else
								{
									USET sn1(patternsSupports[n1].begin(), patternsSupports[n1].end());//kiwi2, change to LSET
									USET dif;//kiwi2, change to LSET
									set_difference(sn2.begin(),sn2.end(),sn1.begin(),sn1.end(),insert_iterator<USET >(dif,dif.begin()));//kiwi2, change to LSET
									if(dif.empty()) 
									{
										patternsSupports.erase(patternsSupports.begin() + n2);
										patterns.erase(patterns.begin() + n2);
										n2--;
										break;
									}
								}
							}
						}
					}
				}
				break;
				case 4: //check if two supporting sets for pattern1 and pattern2 equal or not, if equal, remove one of them
					//theoretically, should remove the one with smaller average indexes on supporting rows in order to find long patterns, but not done.
				{
					for(int n2 = 1; n2 < patterns.size(); n2++) //starting with the 2nd element
					{
						if(n2 >= m_k) //reached m_k
							break;
						for(int n1 = n2 -1; n1 >= 0; n1 --) //check backwards, for better efficiency
						{
							if(patterns[n2].first == patterns[n1].first) //if same support, they must equal, otherwise, not subset of each other
								//in this case, since the last item of the two patterns are equal, remove either pattern would be ok
							{
								if(patternsSupports[n2] == patternsSupports[n1])
								{
									patternsSupports.erase(patternsSupports.begin() + n2);
									patterns.erase(patterns.begin() + n2);
									n2--; //the new one taking the place need to be checked too
									break;
								}
							}
							else //if supports are not equal, the rest ones cannot be equal either (they have bigger supports)
								break;
						}
					}
				}
				break;
				case 5: //check subset relationship of two supporting sets, remove the pattern corresponding to the subset
				{
					for(int n2 = 1; n2 < patterns.size(); n2++) //starting with the 2nd element
					{
						if(n2 >= m_k) //reached m_k
							break;
						USET sn2(patternsSupports[n2].begin(), patternsSupports[n2].end()); //kiwi2, change to LSET
						for(int n1 = n2 - 1; n1 >= 0; n1 --)
						{
							if(patterns[n2].first == patterns[n1].first) //if same support, they must equal, otherwise, not subset of each other
							//in this case, since the last item of the two patterns are equal, remove either pattern would be ok
							{
								if(patternsSupports[n2] == patternsSupports[n1])
								{
									patternsSupports.erase(patternsSupports.begin() + n2);
									patterns.erase(patterns.begin() + n2);
									n2--; //the new one taking the place need to be checked too
									break;
								}
							}
							else
							{
								USET sn1(patternsSupports[n1].begin(), patternsSupports[n1].end());//kiwi2, change to LSET
								USET dif;//kiwi2, change to LSET
								set_difference(sn2.begin(),sn2.end(),sn1.begin(),sn1.end(),insert_iterator<USET>(dif,dif.begin()));//kiwi2, change to LSET
								if(dif.empty()) 
								{
									patternsSupports.erase(patternsSupports.begin() + n2);
									patterns.erase(patterns.begin() + n2);
									n2--;
									break;
								}
							}
						}
					}
				}
				break;
			}

			if(patterns.size() > m_k)
			{
				patterns.resize (m_k);
				patternsSupports.resize (m_k);
			}
			else if(realSize >= m_k && m_deficit < m_k - patterns.size()) 
				//realSize > m_k guarantees, m_k - patterns.size() is from the removal of subsets
				//otherwise, say, the last level, realSize = candidates.size() < m_k, then m_deficit can be m_k
				m_deficit = m_k - patterns.size();
		}
		
		//patternSupport
		str.Format(_T("%d"), level);
		output += "Level " + str + ":\r\n";
		int patternSupport = 0;
		for (int y = 0; y < patterns.size(); y++)
			patternSupport += patterns[y].first;
		str.Format(_T("%d"), patternSupport);
		output += "Pattern support = " + str + "\r\n";

		//now, only patterns and patternsSupport left, they'd be useful for the next level
				
		//insert kpatterns for this level into allPatterns
		if(patterns.size() > 0)
		{
			for (int ee = 0; ee < patterns.size(); ee++)
			{
				LROW s;
				int pointer = patterns[ee].second / columnSize;
				short item = patterns[ee].second % columnSize;
				s.push_back (ee); //index for this pattern
				s.push_back(pointer); //pointer to previous level
				s.push_back(item); //current item
				s.push_back (patterns[ee].first); //support
				kpatterns.push_back(s);
					
				patterns[ee].second = item; //now, in patterns, each row's second column is the item itself
			}
			allPatterns.push_back (kpatterns);
			kpatterns.clear();
		
							
			//get running time information for output
			levelFinish = clock();
			runTime = (double)(levelFinish - levelStart) / CLOCKS_PER_SEC;
			str = gcvt(runTime, 5, charStr);
			output += "Run time = " + str + "\r\n\r\n";
		
		}
		
		else
		//get running time information for output
		{
			levelFinish = clock();
			runTime = (double)(levelFinish - levelStart) / CLOCKS_PER_SEC;
			str = gcvt(runTime, 5, charStr);
			output += "Run time = " + str + "\r\n\r\n";
		    break;
		}
		
	}

	finish = clock();
	m_clusteringTime = (double)(finish - start) / CLOCKS_PER_SEC;
	str = gcvt(m_clusteringTime, 5, charStr);
	output += "----------------------------------\r\nTotal Run time in seconds = " + str;
	//free memory, majorly candidates and lastcandidates
	patterns.clear(); //can I use this to free memory? if not, using loop impossible after .clear(), say, candidates
	patternsSupports.clear();

	//write to file
	CString fileName = m_dataSource;
	fileName.Replace(".txt", "");
	
	//if m_timestampInFileName, prepare date and time info to write
	if(m_timestampInFileName)
	{
		_strdate(charStr);
		str = charStr;
		_strtime( charStr);
		str +=  charStr;
		str.Remove('/');
		str.Remove(':');
	}
	else
		str = "";

	writeFile(m_writeToDir + fileName + str + ".tim", output);
	
	output = "";

	//write allPatterns to file;
	//writeFile(m_writeToDir + fileName + str + ".pat", toStringAllPatterns());

	if(allPatterns.empty())
		return;

	m_patternSource = fileName + str + ".pat";

	CString totalName = m_writeToDir + fileName + str + ".pat";
	ofstream myfile (totalName);
  
	if (myfile.is_open())
	{
		//produce the first line
		CString str, firstLine;
		firstLine = "-level 0-";
		for (int p = 1; p < allPatterns.size(); p++)
		{
			str.Format(_T("%d"), p);
			str = "\t-level " + str + "-"; 
			firstLine += str;
		}
		firstLine += "\n";
		LPCTSTR line = firstLine;
		myfile << line;

		//produce the String for all the patterns
		CString str0, str1, str2, str3, lineStr;
		for (int i = 0; i < m_k; i++)
		{
			lineStr = "";
			for(int j = 0; j < allPatterns.size(); j++)
			{
				if(i < allPatterns[j].size())
				{
					str0.Format(_T("%d"), allPatterns[j][i][0]);
					str1.Format(_T("%d"), allPatterns[j][i][1]);
					str2.Format(_T("%d"), allPatterns[j][i][2]);
					str3.Format(_T("%d"), allPatterns[j][i][3]);
					lineStr += str0 + ";" + str1 + ";" + str2 + ";" + str3 + "\t";
				}
				else //need here to take the space for that level
				{
					str0 = "N";
					lineStr += str0 + "\t";
				}
			}
			lineStr.SetAt(lineStr.GetLength() - 1, '\n');
			LPCTSTR line = lineStr;
			myfile << line;
		}
		myfile.close();
	}
	else 
		m_show = "Unable to open file";

	////////////////////
	//update the query zone
	m_queryTopK = 0;
	for (int q = 1; q < allPatterns.size(); q++) //q = 0 is meaningless
		m_queryTopK += allPatterns[q].size();
	
	m_queryLevelHigh = allPatterns.size();
	m_queryRowIndexHigh = m_k;
	UpdateData(false);
}

void CKiwiDlg::OnClear() 
{
	UpdateData(true);
	m_show = "";
	UpdateData(false);

}

void CKiwiDlg::OnPreprocessing() 
{
	UpdateData(true);
	CString output = "";
	//start reading data
	if(m_dataSource == "")
	{
		m_show = "Open File Error !";
		UpdateData(false);
		return;
	}
	
	ifstream inFile;
	inFile.open(m_dataSource);
	if(!inFile.is_open())
	{
		m_show = "Open File Error !";
		UpdateData(false);
		return;
	}

	CString fileName = m_dataSource;
	CString noise;
	noise.Format(_T("%d"), m_noise);
	fileName = noise + fileName;
	CString totalName = m_writeToDir + fileName;
	
	ofstream myfile (totalName);

	if (myfile.is_open())
	{
		const signed long MAX_LINE_SIZE = 100000; // suppose MAX_LINE_SIZE = 100000;
		char line[ MAX_LINE_SIZE ]; 

		//processing the first line
		inFile.getline(line, MAX_LINE_SIZE );
		CString lineStr = line;
		lineStr += "\n";
		LPCTSTR theLine = lineStr;
		myfile << theLine;

		CString noise1 = "\t" + noise + "\t"; //two consecutive tabs
		CString noise2 = "\t" + noise + "\n"; //a tab at the end of a line
		
		while(!inFile.eof())
		{
			inFile.getline(line, MAX_LINE_SIZE );
			lineStr = line;
			lineStr += "\n"; //after convert line to lineStr, no "\n" at the end of the lineStr
			lineStr.Replace("\t\t", noise1);
			lineStr.Replace("\t\t", noise1);//if there're > 2 consecutive tabs, after handling the first 2, the pointer moves to the position after, need to run this line again.
			lineStr.Replace("\t\n", noise2);//for noise2, a tab at the end of a line
	
			theLine = lineStr;
			myfile << theLine;
		}
			
		myfile.close();
	}
	
	else 
		m_show = "Unable to open file";
	
}

CString CKiwiDlg::toStringAllPatterns()
{
	CString output = "";
	if(allPatterns.size() > 0)
	{
		//produce the first line
		CString str, firstLine;
		firstLine = "-level 0-";
		for (int p = 1; p < allPatterns.size(); p++)
		{
			str.Format(_T("%d"), p);
			str = "\t-level " + str + "-"; 
			firstLine += str;
		}
		firstLine += "\r\n";

		//produce the String for all the patterns
		CString str0, str1, str2, str3, lineStr;
		for (int i = 0; i < m_k; i++)
		{
			lineStr = "";
			for(int j = 0; j < allPatterns.size(); j++)
			{
				if(i < allPatterns[j].size())
				{
					str0.Format(_T("%d"), allPatterns[j][i][0]);
					str1.Format(_T("%d"), allPatterns[j][i][1]);
					str2.Format(_T("%d"), allPatterns[j][i][2]);
					str3.Format(_T("%d"), allPatterns[j][i][3]);
					lineStr += str0 + ";" + str1 + ";" + str2 + ";" + str3 + "\t";
				}
				else //need here to take the space for that level
				{
					str0 = "N";
					lineStr += str0 + "\t";
				}
			}
			lineStr.SetAt(lineStr.GetLength() - 1, '\r');
			lineStr += "\n";
			output += lineStr;
		}
		output = firstLine + output;
	}
	return output;
}

void CKiwiDlg::OnReadPattern() 
{
	UpdateData(true);
	if(m_patternSource == "" || m_patternSource == ".pat")
	{
		m_show = "Open File Error !";
		UpdateData(false);
		return;
	}

	//initialization
	allPatterns.clear();
	queries.clear();
	queryPatterns.clear();
	queryClusters.clear();
	chosenPatterns.clear();
	chosenClusters.clear();
	mark.clear();
	m_sizeOfQueryPatterns = 0;
	
	//start reading pattern
	clock_t start, finish;
	start = clock();

	if(m_patternSource.Find(".pat") == -1)
	{
		m_show = "Can Only Read *.pat Files !";
		UpdateData(false);
		return;
	}

	ifstream inFile;
	inFile.open(m_patternSource);
	if(!inFile.is_open())
	{
		m_show = "Open File Error !";
		UpdateData(false);
		return;
	}

	const signed long MAX_LINE_SIZE = 100000; // suppose MAX_LINE_SIZE = 100000;
	char line[ MAX_LINE_SIZE ]; 
	char * token; //each token is a pattern like 1; 0; 9; 31; 0
	
	//open source, remove the info line, before, there was no info line
	inFile.getline (line, MAX_LINE_SIZE);

	//remove the first line, but get the information of how many levels in total
	int levels = 0;
	inFile.getline(line, MAX_LINE_SIZE);
	token = strtok(line, "\t");
	if (!token)
		return;
	while (token)
	{
		levels ++;
		token = strtok(NULL, "\t" );
	}

	LROW r;
	LSQUARE s;
	allPatterns = LCUBE(levels, s);
	//now move all the infomation into allPatterns;
	int l = 0; //for level
	int k = 0; //to index k patterns
		
	while(!inFile.eof())
	{
		inFile.getline(line, MAX_LINE_SIZE );
		token = strtok(line, "\t;" );
		if(!token)
			continue;
		m_k = k + 1;
		l = 0;
		while(token )
		{
			if(token[0] == 'N') //if no pattern is available
			{
				l ++; //directly go to the next pattern
				token = strtok(NULL, "\t;" );//and get the next token
			}

			else
			{
				allPatterns[l].push_back(r);
				allPatterns[l][k].push_back (atoi(token));
				
				token = strtok(NULL, "\t;" );
				allPatterns[l][k].push_back (atoi(token));
			
				token = strtok(NULL, "\t;" );
				allPatterns[l][k].push_back (atoi(token));
			
				token = strtok(NULL, "\t;" );
				allPatterns[l][k].push_back (atoi(token));
				
				l ++; //go to the next pattern on the same line
				token = strtok(NULL, "\t;" );
			}
		}
		k++; //to process the next line
	}
		
	inFile.close();
	finish = clock();
	m_clusteringTime = (double)(finish - start) / CLOCKS_PER_SEC;

	//update the query zone
	m_queryTopK = 0;
	for (int q = 1; q < allPatterns.size(); q++) //q = 0 is meaningless
		m_queryTopK += allPatterns[q].size();
	
	m_queryLevelHigh = allPatterns.size();
	m_queryRowIndexHigh = m_k;
	UpdateData(false);	
}

//return (level, rowIndex) pairs satisfying the 3 conditions
void CKiwiDlg::OnQueryGetPatterns() 
{
	UpdateData(true);
	queries.clear();
	queryPatterns.clear();
	queryClusters.clear();
	chosenPatterns.clear();
	chosenClusters.clear();
	mark.clear();

	if(allPatterns.empty())
	{
		m_show = "Read Patterns or Clustering to Get Patterns First !";
		UpdateData(false);
		return;
	}

	clock_t start, finish;
	start = clock();
	//to meet condition 3 and 4
	if(!m_closed)
	{
		for (int level = allPatterns.size() - 1; level >= 0; level --)
		{
			if (level > m_queryLevelHigh-1)
				continue;
			else if (level < m_queryLevelLow-1)
				break;
			else
			{
				for (int rowIndex = 0; rowIndex < allPatterns[level].size(); rowIndex ++)
				{
				//	if (rowIndex < m_queryRowIndexLow)
				//		continue;
					if (rowIndex > m_queryRowIndexHigh - 1)
						break;
					//else if (allPatterns[level][rowIndex][3] < m_queryMinSupport) //meet part of condition 1
					//	break;
					//else
					//to accomodate clusterin2, that is, allPatterns is not ranked by support, remove the above and change to the following, less efficient though
					else if (allPatterns[level][rowIndex][3] >= m_queryMinSupport) //meet part of condition 1
						queries.push_back (make_pair(level, rowIndex));
				}
			}
			if(queries.size() > m_queryTopK) //stop because the # of patterns has been beyond m_queryTopK
				break;
		}
	
		//to meet part of condition 1. Now, the pairs are ordered in the way top-k requires, so we can simply take the top k from them
		if(queries.size() > m_queryTopK)
			queries.resize(m_queryTopK);
	}
	else //note, if for clustering2, the following code is not quite correct, min_support constraint is met, but in checking closed condition, still used the old method
		//i.e., assumed the patterns are sorted by support. If I change the code,
	{
		vector < vector < ULPAIR > > dummy (allPatterns.size());//kiwi2, change to LLPAIR
		//dummy has the same size as allPatterns, but each of them is used to store one level of queries
		//so way, keep the structure of allPatterns to make closed checking more efficient

		for (int level = allPatterns.size() - 1; level >= 0; level --)
		{
			if (level > m_queryLevelHigh-1)
				continue;
			else if (level < m_queryLevelLow-1)
				break;
			else
			{
				for (int rowIndex = 0; rowIndex < allPatterns[level].size(); rowIndex ++)
				{
					//if (rowIndex < m_queryRowIndexLow)
					//	continue;
					if (rowIndex > m_queryRowIndexHigh - 1)
						break;
					//else if (allPatterns[level][rowIndex][3] < m_queryMinSupport) //meet part of condition 1
					//	break;
					//else
					//to accomodate clusterin2, that is, allPatterns is not ranked by support, remove the above and change to the following, less efficient though
					else if (allPatterns[level][rowIndex][3] >= m_queryMinSupport)
						dummy[level].push_back (make_pair(level, rowIndex));
				}
			}
		}
	
		//to meet part of condition 1. keep only top k, and remove empty ones in dummy
		int remaining = m_queryTopK;
		for (int d = dummy.size() - 1; d >= 0; d--)
		{
			if(dummy[d].empty())
			{
				dummy.erase(dummy.begin() + d);
				continue;
			}
			if(dummy[d].size() >= remaining)
			{
				dummy[d].resize(remaining);
				for (int r = d -1; r >= 0; r--)
					dummy.erase(dummy.begin() + r);
				break;
			}
			else
				remaining -= dummy[d].size();
		}

		//now, remove non-closed ones

		//to eliminate non-closed patterns. 
		//for a pattern p, if there is another pattern p' that has higher level (longer), 
		//equal or higher support, and is a superpattern, p should be eliminated
		//for clustering2, still assumed ranked by support (to achieve efficiency), may mistakely include some non-closed patterns, but it's ok, since it's before the final scan, all supports are actually temporary.
		int dummySize = dummy.size();
		for (int e = 0; e < dummySize - 1; e++)
		{
			int dummyESize = dummy[e].size();
			for (int u = dummyESize - 1; u >= 0; u--)
			{
				bool quit = false;
				for(int f = e + 1; f < dummySize; f++)
				{
					if(allPatterns[dummy[e][u].first][dummy[e][u].second][3] > allPatterns[dummy[f][0].first][dummy[f][0].second][3])
						break;
					else
					{
						for (int v = 0; v < dummy[f].size(); v++)
						{
							if(allPatterns[dummy[e][u].first][dummy[e][u].second][3] > allPatterns[dummy[f][v].first][dummy[f][v].second][3])
								break; 
							else 
							{
								if(subPattern(dummy[e][u].first, dummy[e][u].second, dummy[f][v].first, dummy[f][v].second))
								{
									dummy[e].erase(dummy[e].begin() + u);
									quit = true;
									break;
								}
							}
						}
						if(quit)
							break;
					}
				}
			}
		}

		//put dummy into queries		
		for (int l = dummy.size() - 1; l >= 0; l--)
		{
			if(!dummy[l].empty())
			{
				for (int p = 0; p < dummy[l].size(); p++)
					queries.push_back(dummy[l][p]);
			}
		}
		dummy.clear();
	}

	
	m_sizeOfQueryPatterns = queries.size();
	
	finish = clock();
	m_queryTime = (double)(finish - start) / CLOCKS_PER_SEC;
	UpdateData(false);
}

void CKiwiDlg::OnQueryShowPatterns() 
{
	UpdateData(true);
	
	if(m_howToShowPatterns)	//shows the patterns in queries, i.e., the last item for each pattern
	{
		if(queries.empty())
			return;
		CString first, second, outputStr;
		
		for (int i = 0; i < queries.size(); i++)
		{
			first.Format(_T("%d"), queries[i].first);
			second.Format(_T("%d"), queries[i].second);
			outputStr += first + "\t" + second + "\r\n";
		}
		m_show = outputStr;
	}
	else
	{
		//display actual patterns, should not bother queryPatterns, instead, use a new temporary one
		SSQUARE tempPatterns (queries.size());
		for(int x = 0; x < queries.size(); x++)
		{
			int l = queries[x].first; //have to use int instead of unsigned short, because in the following while loop, it may go < 0
			unsigned long r = queries[x].second;
			tempPatterns[x] = SROW (l + 1);
			short current;
	
			while (l >= 0)
			{
				current = allPatterns[l][r][2];
	
				tempPatterns[x][l] = current;
				r = allPatterns[l][r][1];
				l = l - 1;
			}
		}
		if(!tempPatterns.empty())
			m_show = toStringSQUARE(tempPatterns);
		tempPatterns.clear();
	}
	UpdateData(false);
}

//first sorting priority: number of dimensions, second priority: number of rows
bool rankCluster (LROW elem1, LROW elem2)
{
	return elem1[0] > elem2[0] || (elem1[0] == elem2[0] && elem1[1] > elem2[1]); 
}

void CKiwiDlg::OnQueryGetClusters() 
{
	UpdateData(true);
	
	chosenPatterns.clear();
	chosenClusters.clear();
	mark.clear();

	if(seqs.empty())
	{
		m_show = "Read File First !";
		UpdateData(false);
		return;
	}
	
	if(queries.empty())
	{
		m_show = "Get Query Patterns First !";
		UpdateData(false);
		return;
	}

	clock_t start, finish;
	start = clock();
	
	//fill queryPatterns
	queryPatterns = SSQUARE (queries.size());
	for(int x = 0; x < queries.size(); x++)
	{
		int l = queries[x].first; //have to use int instead of unsigned short, because in the following while loop, it may go < 0
		unsigned long r = queries[x].second;
		queryPatterns[x] = SROW (l + 1);
		short current;
	
		while (l >= 0)
		{
			current = allPatterns[l][r][2];
	
			queryPatterns[x][l] = current;
			r = allPatterns[l][r][1];
			l = l - 1;
		}
	}

	//get queryClusters
	queryClusters = USQUARE (queries.size()); //kiwi2, changed to LSQUARE

	for (int s = 0; s < seqs.size(); s++) //for each sequence
	{
		for (int p = 0; p < queries.size(); p++) //for each pattern
		{
			if(subSequence(p, s)) //check if queries[p] is a subsequence of seqs[s]
				queryClusters[p].push_back(s);
		}
	}

	//now, for each cluster, get their pattern info and save in queryPatterns;
	//also, try to extend patterns, but for the time being, only extend patterns with support 2 by LCS algorithm
		
	if(m_extend)
	{
		if(m_useLCS) //if checked, for clusters of size 2, use LCS, the rest use my extend
		{
			for(int x = 0; x < queryClusters.size(); x++)
			{
				if(queryClusters[x].size() == 2)
					queryPatterns[x] = LCS(seqs[queryClusters[x][0]], seqs[queryClusters[x][1]]);
				else
					extend(queryPatterns[x], queryClusters[x]);
			}
		}
		else //if not checked, all use extend. extend is much faster than LCS with good result
		{
			for(int x = 0; x < queryClusters.size(); x++)
				extend(queryPatterns[x], queryClusters[x]);
		}
	}
	else
		m_useLCS = false;
	
	//reorder queryClusters and queryPatterns

	//push_back the pattern size info to each cluster
	//Here, important, the number of clusters could be huge, if we want to save this number,
	//we can use unsigned long, not short or unsigned short!!!!!!!!!!!!!!!!!!!!!!1
	LSQUARE forSort(queryClusters.size());
	for (int k = 0; k < queryClusters.size(); k++)
	{
		forSort[k].push_back(queryPatterns[k].size());
		forSort[k].push_back(queryClusters[k].size());
		forSort[k].push_back(k);
	}
	sort(forSort.begin(), forSort.end(), rankCluster);
	
	SSQUARE queryPatterns2 (forSort.size());
	USQUARE queryClusters2 (forSort.size()); //kiwi2, change to LSQUARE
	//sort queryPatterns and queryClusters
	for (int kk = 0; kk < forSort.size(); kk++)
	{
		queryPatterns2[kk] = queryPatterns[ forSort[kk].back() ];
		queryClusters2[kk] = queryClusters[ forSort[kk].back() ];
	}
	queryPatterns = queryPatterns2;
	queryClusters = queryClusters2;
	queryPatterns2.clear();
	queryClusters2.clear();
	forSort.clear();

	//move redundant information
	//the following code is based on the condition that queryClusters and queryPatterns are correctly sorted
	int queryClustersSize = queryClusters.size();
	for (int m = queryClustersSize - 1; m > 0; m--) //don't have to check the first one
	{
		for (int n = m - 1; n >= 0; n--)
		{
			if(queryClusters[m].size() != queryClusters[n].size() || queryPatterns[m].size() != queryPatterns[n].size())
				break;
			else if(queryClusters[m] != queryClusters[n] || queryPatterns[m] != queryPatterns[n])
				continue;
			else
			{
				queryClusters.erase(queryClusters.begin() + m);
				queryPatterns.erase(queryPatterns.begin() + m);
			}
		}
	}
			
	//get initial info for choose clusters to write or show
	m_numberOfClusters = queryClusters.size();
	m_numberOfMarked = 0;
	m_dimensionalityHigh = queryPatterns[0].size();
	m_dimensionalityLow = queryPatterns.back().size();
	
	m_clusterSizeLow = queryClusters[0].size();
	m_clusterSizeHigh = m_clusterSizeLow;
	for (int c = 1; c < queryClusters.size(); c++)
	{
		if(queryClusters[c].size() < m_clusterSizeLow)
			m_clusterSizeLow = queryClusters[c].size();
		if(queryClusters[c].size() > m_clusterSizeHigh)
			m_clusterSizeHigh = queryClusters[c].size();
	}

	finish = clock();
	m_scanTime = (double)(finish - start) / CLOCKS_PER_SEC;

	//write to file
	if(!(queryClusters.size() > 0 && queryPatterns.size() > 0))
		return;

	CString fileName = m_dataSource;
	fileName.Replace(".txt", "");
	char charStr [20];
	CString str;
	//if m_timestampInFileName, prepare date and time info to write
	if(m_timestampInFileName)
	{
		_strdate(charStr);
		str = charStr;
		_strtime( charStr);
		str +=  charStr;
		str.Remove('/');
		str.Remove(':');
	}
	else
		str = "";

	m_clusterSource = fileName + str + ".clu";

	CString totalName = m_writeToDir + fileName + str + ".clu";
	ofstream myfile (totalName);
  
	if (myfile.is_open())
	{
		CString lineStr, str;
		lineStr = "Pattern file Name = " + m_patternSource + "; ";
		
		str.Format(_T("%d"), m_queryTopK);
		lineStr += "top t = " + str + "; # of rows >= ";

		str.Format(_T("%d"), m_queryMinSupport);
		lineStr += str + "; ";

		//open source, commented out
		/*if(m_closed)
			str = "checked";
		else
			str = "not checked";
		lineStr += "Condition 2: closed checkbox = " + str + "; ";
		*/

		str.Format(_T("%d"), m_queryLevelLow);
		lineStr += "length = " + str + " to ";

		str.Format(_T("%d"), m_queryLevelHigh);
		lineStr += str + "; ";

	//	str.Format(_T("%d"), m_queryRowIndexLow);
	//	lineStr += "Condition 3: Row index from " + str + " to ";

		str.Format(_T("%d"), m_queryRowIndexHigh);
		lineStr += "top l per length = " + str + "; ";

		if(m_extend)
			str = "checked";
		else
			str = "not checked";
		lineStr += "extend = " + str + "; ";

		if(m_useLCS)
			str = "checked";
		else
			str = "not checked";
		lineStr += "LCS = " + str + "\n";

		LPCTSTR line = lineStr;
		myfile << line;

		myfile << "# of columns\tcolumns\trows\n";
	
		for (int i = 0; i < queryPatterns.size(); i++)
		{
			lineStr.Format(_T("%d"), queryPatterns[i].size());
		
			for(int j = 0; j < queryPatterns[i].size(); j++)
			{
				lineStr += "\t";
				str.Format(_T("%d"), queryPatterns[i][j]);
				lineStr += str;
			}

			for(int g = 0; g < queryClusters[i].size(); g++)
			{
				lineStr += "\t";
				str.Format(_T("%d"), queryClusters[i][g]);
				lineStr += str;
			}
			lineStr += "\n";
			LPCTSTR line = lineStr;
			myfile << line;
		}
		
		myfile.close();
	}
	else 
		m_show = "Unable to open file";
		

	//The following is too slow, and can't handle big CStrings, changed to the above
	//(1) need to use \n instead of \r\n
	//(2) need to convert CString to c-string using LPCTSTR line = lineStr;
	//(3) also changed the int->CString conversion, now, don't use buffer anymore
	//writeFile(m_writeToDir + fileName + str + ".clu", toStringAllClusters());

	UpdateData(false);
}


bool rankItem (SROW elem1, SROW elem2)
{
	return (elem1[0] < elem2[0] || (elem1[0] == elem2[0] && elem1[1] < elem2[1])); 
}

//extend is much faster than LCS (only for clusters of size 2)
//now, up to 3 items with full support (of cluster size) between two pattern items are handled. 
//More than that, just added not handled, may be I can just add simply add one
//using extend, closed should be checked, since I also check after the last item in a pattern
void CKiwiDlg::extend(SROW &pattern, UROW cluster) //kiwi2, change UROW to LROW
{
	SROW newPattern = pattern;
	int patternSize = pattern.size();
	int clusterSize = cluster.size();
	int columnsSize = columns.size();
	for (int p = -1; p < patternSize; p++) //if p = 3 2 5, then for 3, 2. -1 and pattern.size() because need to consider items before the first item in the pattern and after the last item in the pattern
	{
		SROW temp(columnsSize, 0); //temp will be used to count the support of each item
		int index1, index2;
		
		for(int c = 0; c < clusterSize; c++) //for each sequence in cluster
		{
			if(p == -1)
				index1 = -1; //since exclusive
			else
				index1 = indexes[cluster[c]][pattern[p]]; //the index in that sequence for 3
			if(p == patternSize - 1)
				index2 = seqs[cluster[c]].size();
			else
				index2 = indexes[cluster[c]][pattern[p + 1]]; //the index in that sequence for 2
			for (int i = index1 + 1; i < index2; i++) //for indexes in that sequence between 3 and 2
				temp[seqs[cluster[c]][i]]++; //support ++ for that item
		}

		SROW symbols; //used to store items with full support, i.e., == cluster size
	
		for (int i = index1 + 1; i < index2; i++) //for indexes in last sequence between 3 and 2
		{
			if(temp[seqs[cluster[clusterSize - 1]][i]] == clusterSize) //if the support count in temp == cluster size
				symbols.push_back (seqs[cluster[clusterSize - 1]][i]); //add the item to symbols
		}
		
		//now, symbols contains all symbols with support equal to the cluster size
		while(true)
		{
			if(symbols.empty()) //if empty, nothing to add, else, find the biggest subsequene of the symbols that is a subsequence of all sequences in the cluster
				break;
			else if (symbols.size() == 1)
			{
				newPattern.insert(newPattern.begin() + p + newPattern.size() - patternSize + 1, symbols[0]);
				break;
			}
			else if (symbols.size() == 2)
			{
				bool all = true; //asuumes all sequences are supersequences of symbols
				for (int j = 0; j < clusterSize - 1; j++) //do not have to check last sequence, since symboles is abstracted from it, must be a subsequence of it
				{
					if (!(subSequence(symbols, cluster[j]))) //can add any item in symbols, since no two items possible
					{
						all = false;
						break;
					}
				}
				if(all)
					newPattern.insert(newPattern.begin() + p + newPattern.size() - patternSize + 1, symbols.begin(), symbols.end());
				else
					newPattern.insert(newPattern.begin() + p + newPattern.size() - patternSize + 1, symbols[0]);
				break;
			}
		
			else if (symbols.size() == 3) // there are 4 situations, 3 2 5, 3 2, 3 5, 2 5, other cases just add 1
			{
				bool all = true;
				SROW dummy = symbols;
				for (int j = 0; j < clusterSize - 1; j++) //do not have to check last sequence, since symboles is abstracted from it, must be a subsequence of it
				{
					if (!(subSequence(dummy, cluster[j]))) //can add any item in symbols, since no two items possible
					{
						all = false;
						break;
					}
				}
				if(all)
					newPattern.insert(newPattern.begin() + p + newPattern.size() - patternSize + 1, dummy.begin(), dummy.end());
				else
				{
					all = true;
					dummy.clear();
					dummy.push_back (symbols[0]);
					dummy.push_back (symbols[1]);
					for (int j = 0; j < clusterSize - 1; j++) //do not have to check last sequence, since symboles is abstracted from it, must be a subsequence of it
					{
						if (!(subSequence(dummy, cluster[j]))) //can add any item in symbols, since no two items possible
						{
							all = false;
							break;
						}
					}
					if(all)
						newPattern.insert(newPattern.begin() + p + newPattern.size() - patternSize + 1, dummy.begin(), dummy.end());
					else
					{
						all = true;
						dummy.clear();
						dummy.push_back (symbols[0]);
						dummy.push_back (symbols[2]);
				
						for (int j = 0; j < clusterSize - 1; j++) //do not have to check last sequence, since symboles is abstracted from it, must be a subsequence of it
						{
							if (!(subSequence(dummy, cluster[j]))) //can add any item in symbols, since no two items possible
							{
								all = false;
								break;
							}
						}
						if(all)
							newPattern.insert(newPattern.begin() + p + newPattern.size() - patternSize + 1, dummy.begin(), dummy.end());
						else
						{
							all = true;
							dummy.clear();
							dummy.push_back (symbols[1]);
							dummy.push_back (symbols[2]);
				
							for (int j = 0; j < clusterSize - 1; j++) //do not have to check last sequence, since symboles is abstracted from it, must be a subsequence of it
							{
								if (!(subSequence(dummy, cluster[j]))) //can add any item in symbols, since no two items possible
								{
									all = false;
									break;
								}
							}
							if(all)
							{
								newPattern.insert(newPattern.begin() + p + newPattern.size() - patternSize + 1, dummy.begin(), dummy.end());
								break;
							}
							else
								newPattern.insert(newPattern.begin() + p + newPattern.size() - patternSize + 1, symbols[0]);
						}
					}
				}
				break;
			}
			else 
			{
				//The following several lines  are a simple solution
				//can be used to replace the following more complex solution
				//faster with a bit worse result, mostly affecting the shorter patterns
			
				//newPattern.insert(newPattern.begin() + p + newPattern.size() - patternSize + 1, symbols[0]);
				//symbols.erase(symbols.begin());
				//break;

				//choose one to add it, then return to process it again. 
				//choose the one, whose latest position is earliest. e.g,
				//4 5 2 3
				//5 2 4 3
				//2 3 4 5
				// 3 and 5 are no good, 2 and 4 are the choices with same priority. 
				// Then, if tied, choose the one with earliest average position. Then 2 is the choise.
				// which is a good choice, and only choise that is able to find another one 3 in the next round.
			
				SROW v;
				SSQUARE stats(symbols.size(), v); 
			
				//symbols is a subsequence of the last sequence
				for (int l = 0; l < symbols.size(); l++)
				{
					stats[l].push_back (l);//0 for latest position
					stats[l].push_back (l);//1 for average (total) position
					stats[l].push_back (symbols[l]);//2 for item
				}
				for (int i = 0; i < clusterSize - 1; i++) //for each sequence except the last one, processed already
				{
					SSQUARE s(symbols.size());
					for(int j = 0; j < symbols.size(); j++)
					{
						s[j].push_back (indexes[cluster[i]][symbols[j]]);//the item's index in the sequence
						s[j].push_back (j);// remember it is the jth item in symbols
					}
					//sort s according to the index
					sort(s.begin(), s.end());
					for (int b = 0; b < s.size(); b++)
					{
						if (b > stats[s[b][1]][0])
							stats[s[b][1]][0] = b;
						stats[s[b][1]][1] += b;
					}
				}
				//now, choose the item
				partial_sort(stats.begin(), stats.begin() + 1, stats.end(), rankItem);
				//now, add the item
				short ITEM = stats[0][2];
				newPattern.insert(newPattern.begin() + p + newPattern.size() - patternSize + 1, ITEM);
				if(stats[0][1] == symbols.size() - 1) //if ITEM's lastest position is the last position of some sequence
					break; //then, no hope to find another one
				else
				{
					//update symbols, which should be a subset of the last sequence
					//now, stats[0][2] contains the item, ITEM, added to newPatterns;
					//in symbols, the part after ITEM will be possibly kept.
					//in this part, every item that has index bigger than the index of ITEM in all sequences of the cluster, 
					//will be in the updated symbols, others will be removed.
					//in the above example, after ITEM = 2, 3 4 5 are candidate symbols, 
					//among them, only 3 will be in symbols, and it is the only one.
					symbols.erase(symbols.begin(), find(symbols.begin(), symbols.end(), ITEM) + 1); //this will remove ITEM and its previous items in symbols
					//symbols will not be empty at the moment, see if part
					for (int e = 0; e < symbols.size(); e++)
					{
						for (int q = 0; q < clusterSize; q++)
						{
							//if any e's index is smaller than ITEM's index, e should be removed
							if(indexes[cluster[q]][symbols[e]] < indexes[cluster[q]][ITEM])
							{
								symbols.erase(symbols.begin() + e);
								e--;
								break;
							}
						}
					}
				}  // if use the simple solution, here to add "*" and "/"
			}
		}
	}
	pattern = newPattern;
}

//check if queries[p] is a subsequence of seqs[s]
bool CKiwiDlg::subSequence(int p, int s)
{
	short current, previous; //for entries in "indexes"
	int l = queries[p].first; //have to use int instead of unsigned short, because in the following while loop, it may go < 0 (not in this case since l <= 1 instead of <= 0, but just to be safe)
	unsigned long r = queries[p].second;
	
	current = indexes[s][allPatterns[l][r][2]];
	
	while (l >= 1)
	{
		r = allPatterns[l][r][1];
		l = l - 1;
		previous = indexes[s][allPatterns[l][r][2]];
		if (previous > current || previous < 0) //if current == -1, previous > current or previous = current = -1, then previous < 0. return false
			return false;
		current = previous;
	}
	return true;
}

//choose clusters and patterns from queryClusters and queryPatterns for writing or show
void CKiwiDlg::OnChooseClusters() 
{
	chosenPatterns.clear();
	chosenClusters.clear();
	UpdateData(true);
	for(int i = 0; i < queryClusters.size(); i++)
	{
		if (!(queryPatterns[i].size() < m_dimensionalityLow || queryPatterns[i].size() > m_dimensionalityHigh || queryClusters[i].size() < m_clusterSizeLow || queryClusters[i].size() > m_clusterSizeHigh))
		{
			chosenPatterns.push_back(queryPatterns[i]);
			chosenClusters.push_back(queryClusters[i]);
		}
	}

	m_numberOfClusters = chosenClusters.size();
	//mark the clusters that have exactly the same row members as one of the previous clusters in the list
	m_numberOfMarked = 0;
	mark = vector<int> (chosenClusters.size(), -1);
	if(m_markRepeats)
	{
		for (int q = chosenClusters.size() - 1; q > 0; q--)
			for(int u = 0; u < q; u++)
			{
				if(chosenClusters[q].size() == chosenClusters[u].size() && chosenClusters[q] == chosenClusters[u])
				{
					mark[q] = u;
					m_numberOfMarked ++;
					break;
				}
			}
	}

	UpdateData(false);
}



//this only writes *.sig in the one-line per cluster format as Obi suggested

void CKiwiDlg::OnWriteClusters() 
{
	UpdateData(true);
	if(chosenClusters.empty())
	{
		m_show = "Select Clusters First !";
		UpdateData(false);
		return;
	}

	if(columns.empty() || rows.empty())
	{
		m_show = "Read Files First !";
		return;
	}
	
	CString fileName = m_dataSource;
	fileName.Replace(".txt", "");
	
	CString str;
	char charStr [20];
	if(m_timestampInFileName)
	{
		_strdate(charStr);
		str = charStr;
		_strtime( charStr);
		str +=  charStr;
		str.Remove('/');
		str.Remove(':');
	}
	else
		str = "";
	
	//writeFile(m_writeToDir + fileName + str + ".sig", toStringClusters());

	CString totalName = m_writeToDir + fileName + str + ".sig";
	ofstream myfile (totalName);
  
	if (myfile.is_open())
	{
		//now, prepare some basic information and parameter settings 
		CString head = "";
		//collect basic information
		CString str;	
	
		head += "File Name = " + m_dataSource + "\n";
		head += "Cluster file Name = " + m_clusterSource + "\n";
		
		str.Format(_T("%d"), m_rows);
		head += "number of rows = " + str + "\n";
		str.Format(_T("%d"), m_cols);
		head += "number of columns = " + str + "\n\n";

		//date and time info
		_strdate( charStr);
		str = charStr;
		str = "Date = " + str + "\nTime = ";
		head += str;
		_strtime( charStr);
		str =  charStr;
		head += str + "\n\n";
	
		str.Format(_T("%d"), m_dimensionalityLow);
		head += "# of columns = " + str + " to ";

		str.Format(_T("%d"), m_dimensionalityHigh);
		head += str + "\n";

		str.Format(_T("%d"), m_clusterSizeLow);
		head += "# of rows = " + str + " to ";

		str.Format(_T("%d"), m_clusterSizeHigh);
		head += str + "\n";

		if(m_markRepeats)
			str = "checked";
		else
			str = "not checked";

		head += "mark repeats = " + str + "\n";

		if(m_actual)
			str = "checked";
		else
			str = "not checked";

		head += "actual = " + str + "\n";

		if(m_removeMarked)
			str = "checked";
		else
			str = "not checked";

		head += "removeMarked = " + str + "\n\n";

		if(m_removeMarked)
		{
			str.Format(_T("%d"), m_numberOfClusters - m_numberOfMarked);
			head += "number of clusters = " + str + "\n";
		}
		else
		{
			str.Format(_T("%d"), m_numberOfClusters);
			head += "number of clusters = " + str + "\n";
		
			str.Format(_T("%d"), m_numberOfMarked);
			head += "marked = " + str + "\n";
		}	
	
		head += "-------------------------------------\n";
	
		LPCTSTR line = head;
		myfile << line;
		
		//prepare for the output string depending on m_actual is checked or not

		CString str0, str1, str2, markStr, clusterInfo, rowInfo, columnInfo;
		
		if(!m_actual)
		{
			for(int i = 0; i < chosenClusters.size(); i++)
			{
				//prepare the cluster summary as the first line
				str0.Format(_T("%d"), i);
			
				str1.Format(_T("%d"), chosenClusters[i].size());
				str2.Format(_T("%d"), chosenPatterns[i].size());
			
				if(mark[i] == -1)
					markStr = "";
				else
				{
					markStr.Format(_T("%d"), mark[i]);
					markStr = " ** This cluster has exactly the same row members as cluster_" + markStr + " **";
					if(m_removeMarked)
						continue;
				}

				clusterInfo = "Cluster_" + str0 + ": " + "# of rows(" + str1 + "); " + "# of columns(" + str2 + "); ";
			
				line = clusterInfo;
				myfile << line;
											
				//prepare the row info
				rowInfo = toStringROW2(chosenClusters[i]);

				//rowInfo.SetAt(rowInfo.GetLength() - 1, ')');
				rowInfo = "rows(" + rowInfo + "); ";
				

				//prepare the column info
				columnInfo = toStringROW2(chosenPatterns[i]);

				//columnInfo.SetAt(columnInfo.GetLength() - 1, ')');

				columnInfo = "columns(" + columnInfo + "); ";

				line = rowInfo;
				myfile << line;

				line = columnInfo;
				myfile << line;

				line = markStr + "\n";
				myfile << line;

			}
		}
	
		else
		{
			for(int i = 0; i < chosenClusters.size(); i++)
			{
				clusterInfo = "";
				rowInfo = "";
				columnInfo = "";
				//prepare the cluster summary as the first line
				str0.Format(_T("%d"), i);
			
				str1.Format(_T("%d"), chosenClusters[i].size());
				str2.Format(_T("%d"), chosenPatterns[i].size());
			
				if(mark[i] == -1)
					markStr = "";
				else
				{
					markStr.Format(_T("%d"), mark[i]);
					markStr = " ** This cluster has exactly the same row members as cluster_" + markStr + " **";
					if(m_removeMarked)
						continue;
				}

				clusterInfo = "Cluster_" + str0 + ": " + "# of rows(" + str1 + "); " + "# of columns(" + str2 + "); ";
			
						
					
				//prepare the row info
				for (int g = 0; g < chosenClusters[i].size(); g++)
					rowInfo += rows[chosenClusters[i][g]] + ",";
			
				//rowInfo.Delete(rowInfo.GetLength() - 2, 2);
				rowInfo.SetAt(rowInfo.GetLength() - 1, ')');



				//prepare the column info
			
				for (int t = 0; t < chosenPatterns[i].size(); t++)
					columnInfo += columns[chosenPatterns[i][t]] + ",";

				//columnInfo.Delete(columnInfo.GetLength() - 2, 2);
				columnInfo.SetAt(columnInfo.GetLength() - 1, ')');

				line = clusterInfo;
				myfile << line;
				
				line = "rows(" + rowInfo + "; ";
				myfile << line;

				line = "columns(" + columnInfo + "; ";
				myfile << line;
				
				line = markStr + "\n";
				myfile << line;
			}
		}
		myfile.close();
	}
	else 
		m_show = "Unable to open file";
	
	UpdateData(false);
}


void CKiwiDlg::OnShowClusters() 
{
	UpdateData(true);
	
	if(chosenClusters.empty())
	{
		m_show = "Select Clusters First !";
		UpdateData(false);
		return;
	}

	if(chosenClusters.size() > 5000)
		m_show = "For fast display, maximum of 5000 clusters are allowed. Please write to disk!\r\n";
	else
		m_show = toStringClusters();

	UpdateData(false);
}

//used to display chosen clusters
CString CKiwiDlg::toStringClusters()
{
	UpdateData(true);
	
	//prepare for the output string depending on m_actual is checked or not
	CString output = "";
	if(chosenClusters.empty())
		return output;
	if(columns.empty() || rows.empty())
	{
		m_show = "Read Files First !";
		return output;
	}

	CString str0, str1, str2, markStr, clusterInfo, rowInfo, columnInfo;
		
		if(!m_actual)
		{
			for(int i = 0; i < chosenClusters.size(); i++)
			{
				//prepare the cluster summary as the first line
				str0.Format(_T("%d"), i);
			
				str1.Format(_T("%d"), chosenClusters[i].size());
				str2.Format(_T("%d"), chosenPatterns[i].size());
			
				if(mark[i] == -1)
					markStr = "";
				else
				{
					markStr.Format(_T("%d"), mark[i]);
					markStr = " ** This cluster has exactly the same row members as cluster_" + markStr + " **";
					if(m_removeMarked)
						continue;
				}

				clusterInfo = "Cluster_" + str0 + ": " + "# of rows(" + str1 + "); " + "# of columns(" + str2 + "); ";
			
				output += clusterInfo;
											
				//prepare the row info
				rowInfo = toStringROW2(chosenClusters[i]);
			//	rowInfo.SetAt(rowInfo.GetLength() - 1, ')');

				rowInfo = "rows(" + rowInfo + "); ";
				

				//prepare the column info
				columnInfo = toStringROW2(chosenPatterns[i]);
				//columnInfo.SetAt(columnInfo.GetLength() - 1, ')');

				columnInfo = "columns(" + columnInfo + "); ";

				output += rowInfo;
				output += columnInfo;
				
				output += markStr + "\r\n";

			}
		}
	
		else
		{
			for(int i = 0; i < chosenClusters.size(); i++)
			{
				clusterInfo = "";
				rowInfo = "";
				columnInfo = "";
				//prepare the cluster summary as the first line
				str0.Format(_T("%d"), i);
			
				str1.Format(_T("%d"), chosenClusters[i].size());
				str2.Format(_T("%d"), chosenPatterns[i].size());
			
				if(mark[i] == -1)
					markStr = "";
				else
				{
					markStr.Format(_T("%d"), mark[i]);
					markStr = " ** This cluster has exactly the same row members as cluster_" + markStr + " **";
					if(m_removeMarked)
						continue;
				}

				clusterInfo = "Cluster_" + str0 + ": " + "# of rows(" + str1 + "); " + "# of columns(" + str2 + "); ";
			
						
					
				//prepare the row info
				for (int g = 0; g < chosenClusters[i].size(); g++)
					rowInfo += rows[chosenClusters[i][g]] + ",";
			
				//rowInfo.Delete(rowInfo.GetLength() - 2, 2);

				rowInfo.SetAt(rowInfo.GetLength() - 1, ')');


				//prepare the column info
			
				for (int t = 0; t < chosenPatterns[i].size(); t++)
					columnInfo += columns[chosenPatterns[i][t]] + ",";

				//columnInfo.Delete(columnInfo.GetLength() - 2, 2);
				columnInfo.SetAt(columnInfo.GetLength() - 1, ')');

				output += clusterInfo;
								
				output += "rows(" + rowInfo + "; ";
				
				output += "columns(" + columnInfo + "; ";
								
				output += markStr + "\r\n";
				
			}
		}
	//now, prepare some basic information and parameter settings 
	char charStr [20];
	CString head = "";
	//collect basic information
	CString str;	
	head += "File Name = " + m_dataSource + "\r\n";
	head += "Cluster file Name = " + m_clusterSource + "\r\n";
		
	str.Format(_T("%d"), m_rows);
	head += "number of rows = " + str + "\r\n";
	str.Format(_T("%d"), m_cols);
	head += "number of columns = " + str + "\r\n\r\n";

	//date and time info
	_strdate( charStr);
	str = charStr;
	str = "Date = " + str + "\r\nTime = ";
	head += str;
	_strtime( charStr);
	str =  charStr;
	head += str + "\r\n\r\n";
	str.Format(_T("%d"), m_dimensionalityLow);
	head += "# of columns = " + str + " to ";

	str.Format(_T("%d"), m_dimensionalityHigh);
	head += str + "\r\n";

	str.Format(_T("%d"), m_clusterSizeLow);
	head += "# of rows = " + str + " to ";

	str.Format(_T("%d"), m_clusterSizeHigh);
	head += str + "\r\n";

	if(m_markRepeats)
		str = "checked";
	else
		str = "not checked";

	head += "mark repeats = " + str + "\r\n";

	if(m_actual)
		str = "checked";
	else
		str = "not checked";

	head += "actual = " + str + "\r\n";
	if(m_removeMarked)
		str = "checked";
	else
		str = "not checked";

	head += "removeMarked = " + str + "\r\n\r\n";

	if(m_removeMarked)
	{
		str.Format(_T("%d"), m_numberOfClusters - m_numberOfMarked);
		head += "number of clusters = " + str + "\r\n";
	}
	else
	{
		str.Format(_T("%d"), m_numberOfClusters);
		head += "number of clusters = " + str + "\r\n";
	
		str.Format(_T("%d"), m_numberOfMarked);
		head += "marked = " + str + "\r\n";
	}	
	
	head += "-------------------------------------\r\n";
	output = head + output;
	
	return output;	
}

void CKiwiDlg::OnHelp() 
{
	/*
	UpdateData(true);

	//reading help.txt
	
	ifstream inFile;
	inFile.open("help.txt");
	if(!inFile.is_open())
	{
		m_show = "Open File Error !";
		UpdateData(false);
		return;
	}

	const signed long MAX_LINE_SIZE = 100000; // suppose MAX_LINE_SIZE = 100000;
	char line[ MAX_LINE_SIZE ]; 

	m_show = "";
	while(!inFile.eof())
	{
		inFile.getline(line, MAX_LINE_SIZE );
		m_show = m_show + line + "\r\n";
	}
	UpdateData(false);
	*/

	UpdateData(true);
	m_show = "1. Read input\r\n\r\n";
	m_show += "Main function:\r\n";
	m_show += "Read input: reads input data and transforms it into a sequence database.\r\n";
	m_show += "The input data should be in the following format with tabs as delimiters.\r\n";
	m_show += "\r\n";
	m_show += "columns\tcol1\tcol2\r\n";
	m_show += "row1\t1.2343\t2.422\r\n";
	m_show += "row2\t-0.523\t10.234\r\n";
	m_show += "\r\n";
	m_show += "Parameters:\r\n";
	m_show += "from: specifies which file to read, e.g., human_su_uniprot_matrix.txt.\r\n";
	m_show += "\r\n";
	m_show += "length >=: specifies a threshold (2 by default) for the minimum length of each row.\r\n";
	m_show += "\r\n";
	m_show += "noise as: specifies which entry value should be considered as noise, e.g., 0.\r\n";
	m_show += "\r\n";
	m_show += "Asscociated functions:\r\n";
	m_show += "Show rows: shows the row ids and their corresponding actual names.\r\n";
	m_show += "\r\n";
	m_show += "Show columns: shows the column ids and their corresponding actual names.\r\n";
	m_show += "\r\n";
	m_show += "Show seq: shows the sequence specified by parameter \"id\".\r\n";
	m_show += "It is a sequence of column ids with their corresponding column values sorted in increasing order.\r\n";
	m_show += "Enter \"-1\" to display all the sequences, which would take long if the input data is large.\r\n";
	m_show += "--------------------------------------------------\r\n\r\n";
	m_show += "2. Find patterns\r\n\r\n";
	m_show += "Main function:\r\n";
	m_show += "Find patterns (*.pat): discovers significant patterns and write the output to *.pat.\r\n";
	m_show += "It is the most time consuming part of the program.\r\n";
	m_show += "\r\n";
	m_show += "Parameters:\r\n";
	m_show += "k: specifies how many patterns you wish to keep on each level.\r\n";
	m_show += "A bigger k usually leads to better results with longer running time.\r\n";
	m_show += "\r\n";
	m_show += "w: specifies the width of the w-region.\r\n";
	m_show += "The smaller the w value, the shorter the running time.\r\n";
	m_show += "If k is big enough, a bigger w leads to better results and a big enough w will return the complete set of sequential patterns. However, it is rarely the case.\r\n";
	m_show += "For a fixed k, there is usually a range of w values that can lead to better results.\r\n";
	m_show += "You may need to experimentally decide the w value to specify.\r\n";
	m_show += "A useful heuristic is to choose a small k, such as 1000, then vary w and observe the longest patterns discovered, as shown in the second edit box for parameter \"length\" in \"Query interesting patterns\".\r\n";
	m_show += "Choose the w value leading to the biggest observation.\r\n";
	m_show += "\r\n";
	m_show += "min_row: specifies the smallest number of rows a cluster (2 by default) has to contain.\r\n";
	m_show += "A bigger min_row could result in shorter running time, but in the application of gene expression analysis, a cluster of 2 genes coexpressing in many experiments is very meaningful.\r\n";
	m_show += "remove redundancy: improves the quality of the k patterns on each level by removing some unpromising patterns.\r\n";
	m_show += "There are 6 choices, 0 ~ 5, the bigger the number, the slower. We suggest to choose 0 (not used) or 1 (better results).\r\n";
	m_show += "\r\n";
	m_show += "Associated functions:\r\n";
	m_show += "Show patterns: shows all the discovered patterns that are currently in memory.\r\n";
	m_show += "Patterns are organized in linked lists. Not very useful for domain users.\r\n";
	m_show += "\r\n";
	m_show += "Read patterns: reads previously saved *.pat from the specified location.\r\n";
	m_show += "--------------------------------------------------\r\n\r\n";
	m_show += "3. Query intersting patterns\r\n";
	m_show += "\r\n";
	m_show += "Main function:\r\n";
	m_show += "Query patterns: queries the interesting patterns satisfying the specified conditions.\r\n";
	m_show += "By doing so, we save time for the next step \"Find clusters\" since we only need to scan the database for the interesting patterns.\r\n";
	m_show += "\r\n";
	m_show += "Parameters:\r\n";
	m_show += "top t: specifies the top t interesting patterns from all the discovered patterns.\r\n";
	m_show += "Patterns are ranked according to the first priority of pattern length and second priority of number of supporting rows.\r\n";
	m_show += "# of rows >=: specifies the minimum number of supporting rows (2 by default) each pattern has to contain.\r\n";
	m_show += "length: specifies the range for pattern length, i.e., number of columns.\r\n";
	m_show += "For example, from 6 to 10 means we are only interested in patterns with length 6 to 10.\r\n";
	m_show += "\r\n";
	m_show += "top l per length: specifies the top l patterns for each pattern length.\r\n";
	m_show += "Patterns for a fixed length are ranked according to a ranking statistic, intuitively, the number of supporting rows. \r\n";
	m_show += "For example, if top 100 is specified, given that pattern length is 6 to 10, then the total number of interesting patterns will be at most 100 x 5 = 500.\r\n";
	m_show += "\r\n";
	m_show += "Associated functions:\r\n";
	m_show += "Show queried patterns: shows the queried interesting patterns.\r\n";
	m_show += "There are two formats available, real patterns or their indexes, controled by the checkbox. Not very useful for domain users. \r\n";
	m_show += "--------------------------------------------------\r\n\r\n";
	m_show += "4. Find clusters\r\n";
	m_show += "\r\n";
	m_show += "Main function:\r\n";
	m_show += "Find clusters *.clu: Finds the supporting rows for each interesting pattern to form clusters and write them to *.clu.\r\n";
	m_show += "This is done by scanning the sequence database. Redundant clusters with exactly the same rows and columns as some other cluster will be removed. \r\n";
	m_show += "Clusters will be re-ordered according to the first priority of # of columns (dimensionality), and second priority of # of rows (cluster size).\r\n";
	m_show += "\r\n";
	m_show += "Parameters:\r\n";
	m_show += "extend: if checked (usually), extends the patterns using a heuristic.\r\n";
	m_show += "LCS: if checked (usually) for clusters of size 2, use LCS, a dynamic programming approach for longest common subsequence, to extend the patterns.\r\n";
	m_show += "\r\n";
	m_show += "Associated functions:\r\n";
	m_show += "Read clusters: reads previously saved *.clu from the specified location.\r\n";
	m_show += "--------------------------------------------------\r\n\r\n";
	m_show += "5. Select significant clusters\r\n";
	m_show += "\r\n";
	m_show += "Main function:\r\n";
	m_show += "Select clusters: selects the significant clusters satisfying the specified conditions.\r\n";
	m_show += "\r\n";
	m_show += "Parameters:\r\n";
	m_show += "mark repeats: if checked (usually), marks repeating clusters.\r\n";
	m_show += "Repeating clusters are ones with the same rows but not the same columns, in particular, with fewer or equal number of columns.\r\n";
	m_show += "\r\n";
	m_show += "# of cols: specifies the range for # of columns for clusters.\r\n";
	m_show += "\r\n";
	m_show += "# of rows: specifies the range for # of rows for clusters.\r\n";
	m_show += "\r\n";
	m_show += "Associated functions:\r\n";
	m_show += "Write clusters(*.sig): writes the selected clusters to disk as *.sig.\r\n";
	m_show += "\r\n";
	m_show += "Show clusters: shows the selected clusters.\r\n";
	m_show += "\r\n";
	m_show += "The two parameters \"actual\" and \"remove marked\" specify how the selected clusters are written or shown.\r\n";
	m_show += "Check \"actual\" (usually) if you want to use the actual row and column names instead of their ids.\r\n";
	m_show += "This is useful in distinguishing among rows or columns with the same actual names.\r\n";
	m_show += "Check \"remove marked\" (usually) if you are not interested in repeating clusters.\r\n";
	m_show += "--------------------------------------------------\r\n\r\n";
	m_show += "Misc.\r\n";
	m_show += "\r\n";
	m_show += "timestamp: if checked, adds date & time info to the file names for *.pat, *.clu and *.sig.\r\n";
	m_show += "This way, different executions will generate different files, instead of over-write previous ones.\r\n";
	m_show += "\r\n";
	m_show += "write to: specifies which directory you want to write the output files.\r\n";
	m_show += "By default, they are written to the same directory as the executable.\r\n";
	m_show += "\r\n";
	m_show += "Preprocessing: processes files leaving empty between two tabs for missing value or noise.\r\n";
	m_show += "Such files need to be preprocessed only once.\r\n";
	m_show += "Enter the file name, enter the noise value, then click preprocessing.\r\n";
	m_show += "The processed file will be saved as noise + fileName.\r\n";
	m_show += "Say, human_affy_gene_matrix.txt will be saved as 0human_affy_gene_matrix.txt if 0 is chosen as the noise.\r\n";
	m_show += "Next time when reading the file, enter 0 for noise, then 0 entries will be removed.\r\n";
	m_show += "\r\n";
	m_show += "Doubling: provides a simple way of capturing GOPSMs (generalized OPSMs), say, for modelling anti-correlation.\r\n";
	m_show += "It basically doubles the input sequence database with another set of reversed sequences. It can be used after \"Read input\" and before \"Find paterns\" (less efficient).\r\n";
	m_show += "It can also be used after \"Query patterns\" and before \"Find clusters\".\r\n";
	m_show += "\r\n";
	m_show += "Shrink *.clu: shrinks *.clu by removing useless clusters.\r\n";
	m_show += "First, select clusters (specifying the \"# of cols\" and \"# of rows\" conditions, leaving \"mark repeats\" unchecked), then click Shrink *.clu.\r\n";
	m_show += "A new file with file name = previous file name + shrink + possible timestamp.clu will be saved.\r\n";
	m_show += "\r\n";
	m_show += "Auto: automates the 5 stages and then write the selected clusters.\r\n";
	m_show += "Parameters need to be specified beforehand.\r\n";
	m_show += "\r\n";
	m_show += "Clear: clears the display window.\r\n";
	m_show += "\r\n";
	m_show += "Help: shows this file in the display window.\r\n";
	m_show += "----------------------------------------------\r\n\r\n";
	m_show += "Note:\r\n";
	m_show += "1. This version has a limit of 65535 on the number of rows in the input file. Please contact the authors to get another version if you want to use the program for larger files.\r\n";
	m_show += "\r\n";
	m_show += "2. The source code can be compiled in Microsoft VC++ 6.0 or above. Due to a known bug in dbgheap.c, we suggest you to compile release versions only.\r\n";

	UpdateData(false);

}

//to check in dummy, if a pattern is a subpattern of another pattern
bool CKiwiDlg::subPattern(int li, int ri, int lj, int rj)
{
	while(true)
	{
		if(lj < li) //no way to be a subpattern, doesn't even need to continue
			return false;
		if (li == -1) //all items in pattern i has found properly matching items in pattern j
			return true;
		if (allPatterns[li][ri][2] == allPatterns[lj][rj][2])//if match, both retrieve the next
		{
			ri = allPatterns[li][ri][1]; //note the order is important, ri first, then li
			li--;
			rj = allPatterns[lj][rj][1];
			lj--;
		}
		else //if not match, retrieve the next item in pattern j
		{
			rj = allPatterns[lj][rj][1];
			lj--;
		}
	}
}

int factorial (int num)
{
 int result=1;
 for (int i=1; i<=num; ++i)
    result=result*=i;
 return result;
}

void CKiwiDlg::OncalculateProbability() 
{

	//previous version, didn't use pdf
	/*
	UpdateData(true);
	int first = 1;
	for (int i = m_cols; i > m_cols - m_levels; i--)
		first = first * i;
	
	double second = 0;
	for (int j = m_support; j <= m_rows; j ++)
		second += 1.0 * factorial(m_rows) / factorial (j) / factorial(m_rows - j) * pow(1.0 / factorial(m_levels), j) * pow((1 - 1.0 / factorial(m_levels)), (m_rows - j));

	m_probability = first * second;
	UpdateData(false);
	*/
	
	UpdateData(true);
	int m = m_cols;
	int n = m_rows;
	
	int s = m_levels;
	int k = m_support;
	
	double Usk = 0;
	
	double p = 1.0 / factorial(s);
	for (int i = k; i <= n; i++)
		Usk += Stats::Dists::Discrete::Binomial::PDF(p, n, i);

	for (int j = m; j >= m - s + 1; j--)
		Usk = Usk * j;
	
	m_probability = Usk;
	UpdateData(false);
}


//doubling the original dataset with opposite ordered sequences
//data structures involved: rows, seqs, indexes
//need to updated:m_rows
void CKiwiDlg::Ondoubling() 
{
	UpdateData(true);
	vector <CString> rows2 = rows;
	SSQUARE seqs2 = seqs;
	SSQUARE indexes2 = indexes; //
	for(int i = 0; i < seqs2.size(); i++)
	{
		reverse(seqs2[i].begin(), seqs2[i].end());
		seqs.push_back(seqs2[i]);
	}
	
	for(int j = 0; j < rows2.size(); j++)
	{
		CString temp = rows2[j] + "*";
		rows.push_back(temp);
	}

	for(int m = 0; m < indexes2.size(); m++)
	{
		int n = seqs[m].size() - 1; //use n to substract the index
		for(int l = 0; l < indexes2[m].size(); l ++)
		{
			if(indexes2[m][l] != -1)
				indexes2[m][l] = n - indexes2[m][l];
		}
		indexes.push_back (indexes2[m]);
	}
	m_rows = 2 * m_rows;
	UpdateData(false);
}

SROW CKiwiDlg::LCS(SROW str1, SROW str2)
{
	/*
    Pre:  str1 and str2 are CStrings
    Post: returns the length of the longest common subsequence of
          str1 and str2

    Assume str1 and str2 have lengths M and N respectively.  Initialization
    takes time O(M + N), and filling the table (nested loops that do
    M and N iterations respectively) will take time O(MN).  Filling the
    table dominates the running time, which is thus O(MN).

    This analysis does not include constructing the LCS, which is left as
    an exercise (but which doesn't affect the overall running time).
*/

    // The table needs to have a number of rows equal to the size of the
    // first input string plus ine more for the empty string; it needs
    // a number of columns equal to the length of the second string plus
    // one more for the empty string
    int rows(str1.size() + 1), cols(str2.size() + 1), position, seqLength;
    SROW s(cols, 0);
	SSQUARE table(rows, s);
    int i, j;

	
    // Initialization for the base case where a substring of the first 
    // string is compared to the empty string
    //for (i = 0; i < rows; i++) {
	//table[i][0] = 0;
    //}

	// Initialization for the base case where a substring of the second 
    // string is compared to the empty string
    //for (j = 0; j < cols; j++) {
	//table[0][j] = 0;
    //}

    // Fill the table by applying the recursive rule to known cases.
    for (i = 1; i < rows; i++) {
	for (j = 1; j < cols; j++) {
	    // NOTE that the string indices (which haven't been shifted
	    // to account for the empty string case) are one less than
	    // the table indices.
	    if (str1[i-1] != str2[j-1])
		table[i][j] = max(table[i-1][j], table[i][j-1]);
	    else
		table[i][j] = table[i-1][j-1] + 1;
	}
    }

    // Set the return value
    seqLength = table[rows-1][cols-1];

	//*******************************************************************
    //*  Extra Credit Part:  Construct the actual LCS                   *
    //*******************************************************************

    // position keeps track of how many subsequence characters are left to
    // find
    position = seqLength;
    SROW sequence;
    int x = str1.size(); // Keep track of where we are in first string
    int y = str2.size(); // Keep track of where we are in second string
    while (position > 0) {
	if (str1[x-1] == str2[y-1]) { // Remember: string indices are
                                      // off by one from table entries
	    // Insert space for printing
	    //sequence = " " + sequence;
	    // Add the common charater to the LCS
	    // sequence = str1[x-1] + sequence;
		sequence.push_back (str1[x-1]);
        // Update positions: cut out the common character
	    x--;
	    y--;
	    position--;
	}
	// Move back in table in direction that preserves the longest LCS
	else if (table[x-1][y] > table[x][y-1] ) {
	    x--;
	}
	else {
	    y--;
	}
    }

    //return table[rows-1][cols-1];
	reverse(sequence.begin(), sequence.end());
	return sequence;
}

void CKiwiDlg::OnLCS() 
{
	UpdateData(true);
	if(m_seq1 >= 0 && m_seq1 < seqs.size() && m_seq2 >= 0 && m_seq2 < seqs.size())
	{
		clock_t start, finish;
		start = clock();
		SROW s;
		s = LCS(seqs[m_seq1], seqs[m_seq2]);
		m_show += toStringROW(s) + "\r\n";
		m_longestLength = s.size();
		finish = clock();
		m_LCStime = (double)(finish - start) / CLOCKS_PER_SEC;
		//show the longest common sequence in actual sequence format
		if(m_actual)
		{
			CString temp = "";
			for (int t = 0; t < s.size(); t++)
			temp += columns[s[t]] + "\t";
			temp.SetAt(temp.GetLength() - 1, '\r');
			temp += "\n";
			m_show += temp;
		}
	}
	UpdateData(false);	
}

void CKiwiDlg::OnLCSAll() 
{
	UpdateData(true);
	if(seqs.size() < 2)
		return;
	clock_t start, finish;
	start = clock();
	SROW s, longest;
	int longestLength = 0;
	for(int i = 0; i < seqs.size() - 1; i++)
		for(int j = i + 1; j < seqs.size(); j++)
		{
			s = LCS(seqs[i], seqs[j]);
			if(s.size() > longestLength)
			{
				longestLength = s.size();
				longest = s;
				m_seq1 = i;
				m_seq2 = j;
			}
		}
	
	m_show += toStringROW(longest) + "\r\n";
	m_longestLength = longest.size();
	finish = clock();
	m_LCStime = (double)(finish - start) / CLOCKS_PER_SEC;
	if(m_actual)
	{
		CString temp = "";
		for (int t = 0; t < longest.size(); t++)
		temp += columns[longest[t]] + "\t";
		temp.SetAt(temp.GetLength() - 1, '\r');
		temp += "\n";
		m_show += temp;
	}
	UpdateData(false);	
}

void CKiwiDlg::OnAuto() 
{
	//automatically run columns with parameters properly set
	UpdateData(true);
//	clock_t start, finish;
//	start = clock();

	OnReadData();
	OnClustering2();
	OnQueryGetPatterns();
	OnQueryGetClusters();
	OnChooseClusters();
	OnWriteClusters();

//	finish = clock();
//	m_autoTime = (double)(finish - start) / CLOCKS_PER_SEC;
	UpdateData(false);
	
}

//check if vector<int> sub is a subsequence of seqs[sup]
bool CKiwiDlg::subSequence(SROW sub, int sup)
{
	int current, next; //for entries in "indexes"
	current = sub[0];
	current = indexes[sup][sub[0]];
	for (int i = 1; i < sub.size(); i ++)
	{
		next = indexes[sup][sub[i]];
		if (next < current)
			return false;
		current = next;
	}
	return true;
}

//check if vector<int> sub is a subsequence of vector<int> sup
bool CKiwiDlg::subSequence(SROW sub, SROW sup)
{
	SROW p = sup;
	reverse(p.begin(), p.end());
	bool success = 1;
	for (int i = 0; i < sub.size(); i ++)
	{
		while(true)
		{
			if(p.empty())
			{
				success = 0;
				break;
			}

			if(sub[i] == p.back())
			{
				p.pop_back();
				break;
			}
			else
				p.pop_back();
		}
		if(! success)
			return false;
	}
	return true;
}

//check if vector<int> sub is a subsequence of vector<int> sup
bool CKiwiDlg::subSequence(UROW sub, UROW sup)
{
	UROW p = sup;
	reverse(p.begin(), p.end());
	bool success = 1;
	for (int i = 0; i < sub.size(); i ++)
	{
		while(true)
		{
			if(p.empty())
			{
				success = 0;
				break;
			}

			if(sub[i] == p.back())
			{
				p.pop_back();
				break;
			}
			else
				p.pop_back();
		}
		if(! success)
			return false;
	}
	return true;
}

//before, used it in writeFile(m_writeToDir + fileName + str + ".clu", toStringAllClusters());
//now, don't use it
CString CKiwiDlg::toStringAllClusters()
{
	CString output = "Dimensionality\tDimensions\tGene indexes\r\n";
	if(queryClusters.size() > 0 && queryPatterns.size() > 0)
	{
		CString lineStr, str;
		
		for (int i = 0; i < queryPatterns.size(); i++)
		{
			lineStr.Format(_T("%d"), queryPatterns[i].size());
			
			for(int j = 0; j < queryPatterns[i].size(); j++)
			{
				str.Format(_T("%d"), queryPatterns[i][j]);
				lineStr += "\t" + str;
			}

			for(int g = 0; g < queryClusters[i].size(); g++)
			{
				str.Format(_T("%d"), queryClusters[i][g]);
				lineStr += "\t" + str;
			}
			lineStr += "\r\n";
			output += lineStr;
		}
	}
	return output;
}

void CKiwiDlg::OnReadClusters() 
{
	UpdateData(true);
	if(m_clusterSource == "" || m_clusterSource == ".clu")
	{
		m_show = "Open File Error !";
		UpdateData(false);
		return;
	}

	//initialization
	queryPatterns.clear();
	queryClusters.clear();
	chosenPatterns.clear();
	chosenClusters.clear();
	mark.clear();
	
	//start reading pattern
	clock_t start, finish;
	start = clock();

	if(m_clusterSource.Find(".clu") == -1)
	{
		m_show = "Can Only Read *.clu Files !";
		UpdateData(false);
		return;
	}

	ifstream inFile;
	inFile.open(m_clusterSource);
	if(!inFile.is_open())
	{
		m_show = "Open File Error !";
		UpdateData(false);
		return;
	}

	const signed long MAX_LINE_SIZE = 100000; // suppose MAX_LINE_SIZE = 100000;
	char line[ MAX_LINE_SIZE ]; 
	char * token; 
	
	//remove the first line
	inFile.getline(line, MAX_LINE_SIZE);
	inFile.getline(line, MAX_LINE_SIZE);
	
	//now move all the infomation into queryPatterns and queryClusters
	int dim; //for dimensinality of each line
	
	while(!inFile.eof())
	{
		inFile.getline(line, MAX_LINE_SIZE );
		token = strtok(line, "\t" );
		if(!token)
			continue;
		dim = atoi(token);
		SROW s;
		for (int d = 0; d < dim; d++)
		{
			token = strtok(NULL, "\t" );
			s.push_back (atoi(token));
		}
		queryPatterns.push_back(s);

		UROW r; //kiwi2, changed to LROW
		token = strtok(NULL, "\t" );
		while(token)	
		{
			r.push_back (atoi(token));
			token = strtok(NULL, "\t" );
		}
		queryClusters.push_back(r);
	}

	inFile.close();
	finish = clock();
	m_scanTime = (double)(finish - start) / CLOCKS_PER_SEC;

	//update the choosing cluster zone
	m_numberOfClusters = queryClusters.size();
	m_numberOfMarked = 0;
	m_dimensionalityHigh = queryPatterns[0].size();
	
	m_dimensionalityLow = m_dimensionalityHigh;
	m_clusterSizeLow = queryClusters[0].size();
	m_clusterSizeHigh = m_clusterSizeLow;
	for (int c = 1; c < queryClusters.size(); c++)
	{
		if(queryPatterns[c].size() < m_dimensionalityLow)
			m_dimensionalityLow = queryPatterns[c].size();
		if(queryClusters[c].size() < m_clusterSizeLow)
			m_clusterSizeLow = queryClusters[c].size();
		if(queryClusters[c].size() > m_clusterSizeHigh)
			m_clusterSizeHigh = queryClusters[c].size();
	}

	UpdateData(false);	
}

void CKiwiDlg::OnShrinkClu() 
{
	UpdateData(true);
	
	queryClusters = chosenClusters;
	queryPatterns = chosenPatterns;
	
	if(!(queryClusters.size() > 0 && queryPatterns.size() > 0))
		return;

	CString fileName = m_clusterSource;
	fileName.Replace(".clu", "");
	fileName += "shrink";
	char charStr [20];
	CString str;
	//prepare date and time info to write
	if(m_timestampInFileName)
	{
		_strdate(charStr);
		str = charStr;
		_strtime( charStr);
		str +=  charStr;
		str.Remove('/');
		str.Remove(':');
	}
	else
		str = "";

	CString totalName = m_writeToDir + fileName + str + ".clu";
	ofstream myfile (totalName);
  
	if (myfile.is_open())
	{
		CString lineStr, str;
		lineStr = "Cluster file Name = " + m_clusterSource + "; ";
		
		str.Format(_T("%d"), m_dimensionalityLow);
		lineStr += "# of columns = " + str + " to ";

		str.Format(_T("%d"), m_dimensionalityHigh);
		lineStr += str + ";";

		str.Format(_T("%d"), m_clusterSizeLow);
		lineStr += " # of rows = " + str + " to ";

		str.Format(_T("%d"), m_clusterSizeHigh);
		lineStr += str + "\n";
	
		LPCTSTR line = lineStr;
		myfile << line;
		myfile << "# of columns\tColumns\trows\n";
		
		
		for (int i = 0; i < queryPatterns.size(); i++)
		{
			lineStr.Format(_T("%d"), queryPatterns[i].size());
		
			for(int j = 0; j < queryPatterns[i].size(); j++)
			{
				lineStr += "\t";
				str.Format(_T("%d"), queryPatterns[i][j]);
				lineStr += str;
			}

			for(int g = 0; g < queryClusters[i].size(); g++)
			{
				lineStr += "\t";
				str.Format(_T("%d"), queryClusters[i][g]);
				lineStr += str;
			}
			lineStr += "\n";
			LPCTSTR line = lineStr;
			myfile << line;
		}
		myfile.close();
	}
	
	else 
		m_show = "Unable to open file";
}


void CKiwiDlg::OnWriteClusters2() 
{
	// TODO: Add your control notification handler code here
	
}



//to transform the sequences into test.data, which is used by gsp


//write clusters, output rows in the chosenCluster, full dimensional space, real expression values.
void CKiwiDlg::OnWriteActualClusters() 
{
	if(queryClusters.empty())
		return;
	UpdateData(true);
	vector<unsigned short> cluster = chosenClusters[0];
	for (int i = 0; i < cluster.size(); i ++)
	{
		if (cluster[i] >= m_rows / 2)
			cluster[i] = cluster[i] - m_rows / 2;
	}

	sort(cluster.begin(), cluster.end());

	if(m_dataSource == "")
	{
		m_show = "Open File Error !";
		UpdateData(false);
		return;
	}
	
	ifstream inFile;
	inFile.open(m_dataSource);
	if(!inFile.is_open())
	{
		m_show = "Open File Error !";
		UpdateData(false);
		return;
	}

	ofstream wfile ("cluster.txt");

	const signed long MAX_LINE_SIZE = 100000; // suppose MAX_LINE_SIZE = 100000;
	char line[ MAX_LINE_SIZE ]; 
	char * token;

	//read in the first line, which contains the columns
	inFile.getline(line, MAX_LINE_SIZE);
	
	int geneIndex = -1; //for rows in the sequences
	int next = 0; //for next geneIndex stored in chosenClusters[0] //Select cluster should always return 1
	while(!inFile.eof())
	{
		CString stringLine = "\n";
		if (next >= cluster.size())
			break;
		inFile.getline(line, MAX_LINE_SIZE );
		token = strtok(line, "\t" ); //the first token contains info about gid
		if(token == NULL) //added this, so, skip any empty lines
			continue;
		token = strtok(NULL, "\t"); //get the next token
		int numberOfTokens = 0;
		while(token )
		{
			if (atof(token) != m_noise)
			{
				stringLine += token;
				stringLine += "\t";
				numberOfTokens ++;
			}
			token = strtok(NULL, "\t"); //get the next token
		}
		
		if (numberOfTokens >= m_seqLengthThreshold)
			geneIndex ++;
		else
			continue;
		if (geneIndex == cluster[next])
		{
			LPCTSTR aline = stringLine;
			wfile << aline;
			next ++;
		}
	}
	wfile.close();
}

//used when w is relatively large, try to be able to capture long patterns as well as patterns with bigger supports.
//the only change is, not to use frequency as the statistic, but m_cols - indexes[item] where item is an item in the w-region. 
//This way, more frequent patterns will be favored, less frequent but appear early ones will be favored too (may produce long patterns).
// also, change vector < ULPAIR > patterns to vector < LLPAIR > patterns;
void CKiwiDlg::OnClustering2() 
{
	UpdateData(true);
	if(seqs.empty())
	{
		m_show = "Read File First !";
		UpdateData(false);
		return;
	}
	
	//initialization
	allPatterns.clear();
	queries.clear();
	queryPatterns.clear();
	queryClusters.clear();
	chosenPatterns.clear();
	chosenClusters.clear();
	mark.clear();
	m_sizeOfQueryPatterns = 0;
	m_deficit = 0;

	if(m_w > columns.size())
		m_w = columns.size();
	
	int columnSize = columns.size();
	
	//open sourced. if subset != 0, automatically choose kplus = k
	if (m_subset != 0)
		m_kplus = m_k; 

	//open source. commented out
	//output will be used for output
	/*
	CString output = "";
	//collect basic information
	char charStr [20];
	CString str;	
	output += "File Name = " + m_dataSource + "\r\n";
	//date and time info
	_strdate( charStr);
    str = charStr;
	str = "Date = " + str + "\r\nTime = ";
	output += str;
	_strtime( charStr);
	str =  charStr;
	output += str + "\r\n\r\n";
	
	str.Format(_T("%d"), m_rows);
	output += "Number of rows = " + str + "\r\n";
	str.Format(_T("%d"), m_cols);
	output += "Number of columns = " + str + "\r\n\r\n";
	
	//for parameter settings
	output += "Parameter setting:\r\n";
	str.Format(_T("%d"), m_noise);
	output += "noise = " + str + "\r\n";
	str.Format(_T("%d"), m_seqLengthThreshold);
	output += "length >= " + str + "\r\n";
	str.Format(_T("%d"), m_numberOfRemoved);
	output += "removed = " + str + "\r\n";
	
	str.Format(_T("%d"), m_k);
	output += "k = " + str + "\r\n";
	str.Format(_T("%d"), m_w);
	output += "w = " + str + "\r\n";
	str.Format(_T("%d"), m_min_row);
	output += "min_row = " + str + "\r\n";

	
	str.Format(_T("%d"), m_memory);
	output += "memory = " + str + "\r\n";
	str.Format(_T("%d"), m_subset);
	output += "remove redundancy = " + str + "\r\n";
	str.Format(_T("%d"), m_kplus);
	output += "kplus = " + str + "\r\n";
	str.Format(_T("%d"), m_deficit);
	output += "deficit = " + str + "\r\n\r\n";
	*/
	
	//Program running information
	//output += "Program running information:\r\n";//open source. commented out
		
	//start running the first level
	clock_t start, finish;//, levelStart, levelFinish;//open source. commented out
	start = clock();
	//levelStart = clock();//open source. commented out
	
	//for 1-item
	int numOfCandidates = columnSize;
	ULPAIR s; //so, k * d is upper bounded by 4294967295//kiwi2, change to LLPAIR
	// each pair initially contians count = 0 and the index of that pair
	//the index can be used to caculate the corresponding item. For the first level, the size of candidates is d
	//and the indexes are column (item) ids themselves.
	//the other levels, the size of candidates is k x d. For each index i, i div d returns a pointer to the pattern in 
	//previous level, and i mod d returns the column (item) id for the current level.
	//and will be attatched with seq id's that support the candidate
	USQUARE candidatesSupports (numOfCandidates); //having the same size as candidates, containing suppporting rows for the candidate of the same index in candidates	//kiwi2, change to LSQUARE
	//candidatesSupports is now used only for the first level. On other levels, count supports first, then choose the top k ones, and then get their supports.
	//saves lots of memory, a lot faster than before
	//now, only used for the first level
	vector < LLPAIR > patterns(numOfCandidates); //support count and store index (for candidates, can be used to derive pointer to previous item and current item index)
	USQUARE patternsSupports;//kiwi2, change to LSQUARE
	
	for (int z = 0; z < patterns.size(); z++)
		patterns[z] = make_pair(0, z);

	//insert seqs of width m_w into candidates and supports to count support
	for (int j = 0; j < seqs.size(); j ++)
		for(int w = 0; w < m_w; w++)
		{
			if(w < seqs[j].size()) //doesn't need this if no noises
			{
				int v = seqs[j][w];
				candidatesSupports[v].push_back(j);
				patterns[v].first += m_cols - w;
			}
		}
	
	//eliminate patterns < min_row
	vector < LLPAIR > okpatterns; //kiwi2, change to LLPAIR
	for (int ok = 0; ok < patterns.size(); ok++)
	{
		if(candidatesSupports[ok].size() >= m_min_row)
			okpatterns.push_back(patterns[ok]);
	}
	patterns = okpatterns;
	okpatterns.clear();
	
	//get realSize
	int realSize = min(m_k, patterns.size());
	
	//partial sort candidates and use the top k indexes to resize supports, here use rankSupport2
	partial_sort(patterns.begin(), patterns.begin() + realSize, patterns.end(), rankSupport2);

	//now, adjust realSize according to memory limitation
	//allowedTotalSupport shoudl be < m_memory (MB) * 1048576 / 2 (each unsigned int takes 2 bytes).
	
	//open source. commented out
	//unsigned long allowedTotalSupport = m_memory * 1048576.0 / 2; //need to put it here since the loops may need it //kiwi2, change to 4
	/*if(m_memory != -1)
	{
		int sup = 0;
		for (int k = 0; k < realSize; k++)
		{
			sup += patterns[k].first;
			if(sup > allowedTotalSupport)
				break;
		}
		if (k + 1 < realSize)
			realSize = k;
	}
	*/

	//first round, do not need to remove subsets, since the possibility is low and candidates are not many
	
	patterns.resize(realSize);
	
	//open source. commented out
	//patternSupport
	/*
	output += "Level 0:\r\n";
	int patternSupport = 0;
	for (int y = 0; y < patterns.size(); y++)
		patternSupport += patterns[y].first;
	str.Format(_T("%d"), patternSupport);
	output += "Pattern support = " + str + "\r\n";
	*/

	
    /////////////still sorted by support//////////////////
	//put support info into patterns
	//for (int c = 0; c < patterns.size(); c++)
	//	patterns[c].first = candidatesSupports[patterns[c].second].size();
	
	////////////runningExample, remove the following one line////////////////
	//now sort patterns
	//sort(patterns.rbegin(),  patterns.rend());
			
	//insert only suppporting rows for top k patterns into patternsSupports
	//for (int c2 = 0; c2 < patterns.size(); c2++)
	//	patternsSupports.push_back(candidatesSupports[patterns[c2].second]);
	
	/////////////still sorted by support////////////////
	/////////////not sorted by support, but leeway, but still put support info into patterns///////////
	for (int c = 0; c < patterns.size(); c++)
	{
		patternsSupports.push_back(candidatesSupports[patterns[c].second]);
		patterns[c].first = patternsSupports[c].size();
	}
	/////////////not sorted by support, but leeway, but still put support info into patterns///////////
	
	candidatesSupports.clear();
	//now, only patterns and patternsSupport left, they'd be useful for the next level
	
	//insert the level 1 patterns into allPatterns
	LSQUARE kpatterns;
	
	if(patterns.empty())
		return;
	
	for (int e = 0; e < patterns.size(); e++)
	{
		LROW s;
		s.push_back (e);
		s.push_back (0); //level 0, no previous level to point to
		s.push_back (patterns[e].second);  // second is the item
		s.push_back (patterns[e].first); // first is the support
		kpatterns.push_back(s);
	}
	
	allPatterns.push_back (kpatterns);
	kpatterns.clear();
	
	//open source. commented out
	//get running time information for output
	/*double runTime;
	levelFinish = clock();
	runTime = (double)(levelFinish - levelStart) / CLOCKS_PER_SEC;
	str = gcvt(runTime, 5, charStr);
	output += "Run time = " + str + "\r\n\r\n";
	*/
	//for 2-item and more
	//patterns will be used for each level, the first column is its support (# of counts)
	//the second column is its index
		
	//Then the top k of them (if k > patterns.size()) will be chosen as the patterns of this level and stored into allPatterns
	//Then get the suppporting rows for each pattern in patterns
	//allPatterns is a global variable, a vector of a vector of pairs
	//For each level, a vector of pairs is used to store the k patterns, a pair is <index for the item of previous level, item for current level>
		
	int level = 0; //for output information, curret level is 1
	while (true) //until support of the most frequent pattern < min_row;
	{
		//levelStart = clock(); //open source, commented out
		level ++;
		numOfCandidates = allPatterns.back().size() * columnSize;
		
		vector < LLPAIR > patterns (numOfCandidates); //stores support count and index for each candidate
		vector <unsigned short> supports (numOfCandidates, 0); //stores support for each candidates, since patterns now does not store supports anymore
		//for kiwi2, changed to the following
		//vector <unsigned long> supports (numOfCandidates, 0); //stores support for each candidates, since patterns now does not store supports anymore
		for (int z = 0; z < patterns.size(); z++)
			patterns[z] = make_pair(0, z);

		//insert relevant data into candidates to count support
		int seqNo;
		int startPos, endPos, candidateIndex;
		for (int f = 0; f < allPatterns.back().size(); f ++) //for each pattern
		{
			for(int p = 0; p < patternsSupports[f].size(); p++)  //for each supporting sequence
			{
				//get the position for the last item of the pattern in the sequence
				//since each column appears exactly once (if noise, at most once) in each sequence
				//short lastItem = patterns[f][1];
				//short posForLastItem = indexes[seqNo][lastItem];
				seqNo = patternsSupports[f][p];
				startPos = indexes[seqNo][allPatterns.back()[f][2]] + 1;
				endPos = min(startPos + m_w, seqs[seqNo].size());
				while(startPos < endPos) //for each pos after posForLastItem
				{
					candidateIndex = f * columnSize + seqs[seqNo][startPos]; //get index in candidates
					patterns[candidateIndex].first += m_cols - startPos; //this is a lot faster than before. before, I push_back the gene id to candidateSupports...
					supports[candidateIndex] ++;
					startPos++;
				}
			}
		}

	
		//eliminate patterns < min_row
		vector < LLPAIR > okpatterns; //kiwi2, change to LLPAIR
		for (int ok = 0; ok < patterns.size(); ok++)
		{
			if(supports[ok] >= m_min_row)
				okpatterns.push_back(patterns[ok]);
		}
		patterns = okpatterns;
		okpatterns.clear();
		supports.clear();

		//get realSize
		if(m_subset) //level >= 1 already
			realSize = min(m_k + m_kplus, patterns.size());
		else
			realSize = min(m_k, patterns.size());
		
		//partial sort patterns, here use rankSupport2
		partial_sort(patterns.begin(), patterns.begin() + realSize, patterns.end(), rankSupport2);
	
		//open source. commented out
		//now, adjust realSize according to memory limitation
		/*if(m_memory != -1)
		{
			int sup = 0;
			for (int kk = 0; kk < realSize; kk++)
			{
				sup += patterns[kk].first;
				if(sup > allowedTotalSupport)
					break;
			}
			if (kk + 1 < realSize)
				realSize = kk;
		}
		*/
		
		patterns.resize(realSize);

		
		//remove patterns with support < min_row //this methods works only after sorting patterns
		//int x;
		//for (x = patterns.size() - 1; x >=0; x--)
		//{
		//	if(patterns[x].first >= m_min_row)
		//		break;
		//}
		//patterns.resize(x + 1);
		
		//while (! patterns.empty())
		//{
		//	if(patterns.back().first < m_min_row)
		//		patterns.pop_back ();
		//	else
		//		break;
		//}

		//insert only suppporting rows for top k patterns into patternsSupports
		//also, store the support info into patterns[i].first, this way, the followoing codes do not have to be changed
		USQUARE temppatternsSupports(patterns.size());//kiwi2, change to LSQUARE
		for (int c = 0; c < patterns.size(); c++)
		{
			int pointer = patterns[c].second / columnSize;
			short item = patterns[c].second % columnSize;
			short lastItem = allPatterns.back()[pointer][2];
			for (int row = 0; row < patternsSupports[pointer].size(); row++)
			{
				//seqNo = patternsSupports[f][p];
				//startPos = indexes[seqNo][patterns[f].second] + 1;
				int seqNo = patternsSupports[pointer][row];
				int dif = indexes[seqNo][item] - indexes[seqNo][lastItem];
				if (dif > 0 && dif <= m_w)
					temppatternsSupports[c].push_back (seqNo);
			}
			patterns[c].first = temppatternsSupports[c].size(); //added this line for clustering2
		}
		
		patternsSupports = temppatternsSupports;
	
		////////////still sorted by support //////////////////
		//now sort patterns
		//vector < LLPAIR > temp = patterns;
		//for (int m = 0; m < temp.size(); m++)
		//	temp[m].second = m;
		
		//sort(temp.rbegin(), temp.rend());
		
		//vector < LLPAIR > tempPatterns = patterns;
		//for (int u = 0; u < temp.size(); u++)
		//{
		//	patterns[u] = tempPatterns[temp[u].second];
		//	patternsSupports[u] = temppatternsSupports[temp[u].second];
		//}
		//temp.clear();
		//tempPatterns.clear();
		////////////still sorted by support //////////////////
		
		temppatternsSupports.clear();


		//now, patterns is as before, as in clustering, stores support and index for each pattern
		//but not ranked by support, so, need to modify the subset function and querypatterns later on
		//because they are designed based on the fact that ordered by support
		
		//now, remove subsets
		if(m_subset)//m_subset = 1, 2 or 3. 1 for check only the last item, 2 check back 2, 3 check all
		//the task is to try to get k patterns with no subsets until patterns gets empty
		//now, m_subset can be 4 or 5 too. 4 for equality of two supporting sets no matter items match or not
		//5 is for subset relationship of two supporting sets no matter items match or not
		{
			switch(m_subset)
			{
				case 1: //check item2 with item1, possibly remove pattern2 if it item2 == item1 and corresponding suppporting rows of pattern2 is a subset of pattern1
				{
					for(int n2 = 1; n2 < patterns.size(); n2++) //starting with the 2nd element
					{
						if(n2 >= m_k) //reached m_k
							break;
						short item2 = patterns[n2].second % columnSize;
						USET sn2(patternsSupports[n2].begin(), patternsSupports[n2].end()); //kiwi2, change to LSET
						for(int n1 = n2 - 1; n1 >= 0; n1 --)
						{
							short item1 = patterns[n1].second % columnSize;
							if(item2 == item1)
							{
								if(patterns[n2].first == patterns[n1].first) //if same support, they must equal, otherwise, not subset of each other
									//in this case, since the last item of the two patterns are equal, remove either pattern would be ok
								{
									if(patternsSupports[n2] == patternsSupports[n1])
									{
										patternsSupports.erase(patternsSupports.begin() + n2);
										patterns.erase(patterns.begin() + n2);
										n2--; //the new one taking the place need to be checked too
										break;
									}
								}
								//else
								//change to the following for clustering2, based on the observation that: only later patterns will be removed. 
								//If an early pattern has less support, but since it has higher leeway, it must because its last element appears earlier, so, cannot be dominated by a later pattern
								//Since we need to satisfy the subset constraint, so only if an early pattern has higher support, it will dominate an later one
								//after the change, it becomes more efficient becaues the condition is more strict
								else if (patterns[n2].first < patterns[n1].first)
								{
									USET sn1(patternsSupports[n1].begin(), patternsSupports[n1].end());//kiwi2, change to LSET
									USET dif;//kiwi2, change to LSET
									set_difference(sn2.begin(),sn2.end(),sn1.begin(),sn1.end(),insert_iterator<USET>(dif,dif.begin()));//kiwi2, change to LSET
									if(dif.empty()) 
									{
										patternsSupports.erase(patternsSupports.begin() + n2);
										patterns.erase(patterns.begin() + n2);
										n2--;
										break;
									}
								}
							}
						}
					}
				}
				break;
				case 2: //check item2 with item1 and lastItem1
				{
					for(int n2 = 1; n2 < patterns.size(); n2++) //starting with the 2nd element
					{
						if(n2 >= m_k) //reached m_k
							break;
						short item2 = patterns[n2].second % columnSize;
						int ptLastItem2 = patterns[n2].second / columnSize;
						short lastItem2 = allPatterns[level - 1][ptLastItem2][2];

						USET sn2(patternsSupports[n2].begin(), patternsSupports[n2].end()); //kiwi2, change to LSET
						for(int n1 = n2 - 1; n1 >= 0; n1 --)
						{
							short item1 = patterns[n1].second % columnSize;
							if(item2 == item1 || lastItem2 == item1)
							{
								if(patterns[n2].first == patterns[n1].first) //if same support, they must equal, otherwise, not subset of each other
								//in this case, pattern1 has bettern potential to grow longer, so kill pattern2
								{
									if(patternsSupports[n2] == patternsSupports[n1])
									{
										patternsSupports.erase(patternsSupports.begin() + n2);
										patterns.erase(patterns.begin() + n2);
										n2--; //the new one taking the place need to be checked too
										break;
									}
								}
								//else
								//change to the following for clustering2, based on the observation that: only later patterns will be removed. 
								//If an early pattern has less support, but since it has higher leeway, it must because its last element appears earlier, so, cannot be dominated by a later pattern
								//Since we need to satisfy the subset constraint, so only if an early pattern has higher support, it will dominate an later one
								//after the change, it becomes more efficient becaues the condition is more strict
								{
									USET sn1(patternsSupports[n1].begin(), patternsSupports[n1].end());//kiwi2, change to LSET
									USET dif;//kiwi2, change to LSET
									set_difference(sn2.begin(),sn2.end(),sn1.begin(),sn1.end(),insert_iterator<USET >(dif,dif.begin()));//kiwi2, change to LSET
									if(dif.empty()) 
									{
										patternsSupports.erase(patternsSupports.begin() + n2);
										patterns.erase(patterns.begin() + n2);
										n2--;
										break;
									}
								}
							}
						}
					}
				}
				break;
				case 3: //check item2 with all items in pattern1
				{
					for(int n2 = 1; n2 < patterns.size(); n2++) //starting with the 2nd element
					{
						if(n2 >= m_k) //reached m_k
							break;
						vector <int> items2;
						items2.push_back (patterns[n2].second % columnSize);
						int ptItem2 = patterns[n2].second / columnSize;
						for (int le = level - 1; le >= 0; le --)
						{
							items2.push_back (allPatterns[le][ptItem2][2]);
							ptItem2 = allPatterns[le][ptItem2][1];
						}
						USET sn2(patternsSupports[n2].begin(), patternsSupports[n2].end()); //kiwi2, change to LSET
						for(int n1 = n2 - 1; n1 >= 0; n1 --)
						{
							short item1 = patterns[n1].second % columnSize;
							//if find items2 in items1
							if(find(items2.begin(), items2.end(), item1) != items2.end())
							{
								if(patterns[n2].first == patterns[n1].first) //if same support, they must equal, otherwise, not subset of each other
								//in this case, pattern1 has potential to grow longer, so kill pattern2
								{
									if(patternsSupports[n2] == patternsSupports[n1])
									{
										patternsSupports.erase(patternsSupports.begin() + n2);
										patterns.erase(patterns.begin() + n2);
										n2--; //the new one taking the place need to be checked too
										break;
									}
								}
								//else
								//change to the following for clustering2, based on the observation that: only later patterns will be removed. 
								//If an early pattern has less support, but since it has higher leeway, it must because its last element appears earlier, so, cannot be dominated by a later pattern
								//Since we need to satisfy the subset constraint, so only if an early pattern has higher support, it will dominate an later one
								//after the change, it becomes more efficient becaues the condition is more strict
								{
									USET sn1(patternsSupports[n1].begin(), patternsSupports[n1].end());//kiwi2, change to LSET
									USET dif;//kiwi2, change to LSET
									set_difference(sn2.begin(),sn2.end(),sn1.begin(),sn1.end(),insert_iterator<USET >(dif,dif.begin()));//kiwi2, change to LSET
									if(dif.empty()) 
									{
										patternsSupports.erase(patternsSupports.begin() + n2);
										patterns.erase(patterns.begin() + n2);
										n2--;
										break;
									}
								}
							}
						}
					}
				}
				break;
				case 4: //check if two supporting sets for pattern1 and pattern2 equal or not, if equal, remove one of them
					//theoretically, should remove the one with smaller average indexes on supporting rows in order to find long patterns, but not done.
				{
					for(int n2 = 1; n2 < patterns.size(); n2++) //starting with the 2nd element
					{
						if(n2 >= m_k) //reached m_k
							break;
						for(int n1 = n2 -1; n1 >= 0; n1 --) //check backwards, for better efficiency
						{
							if(patterns[n2].first == patterns[n1].first) //if same support, they must equal, otherwise, not subset of each other
								//in this case, since the last item of the two patterns are equal, remove either pattern would be ok
							{
								if(patternsSupports[n2] == patternsSupports[n1])
								{
									patternsSupports.erase(patternsSupports.begin() + n2);
									patterns.erase(patterns.begin() + n2);
									n2--; //the new one taking the place need to be checked too
									break;
								}
							}
							//for clustering2, remove the following two lines, because now it's not ranked by support, it can happen that p1 (3), p3 (4), p2 (3), 
							//else //if supports are not equal, the rest ones cannot be equal either (they have bigger supports)
							//	break;
						}
					}
				}
				break;
				case 5: //check subset relationship of two supporting sets, remove the pattern corresponding to the subset
				{
					for(int n2 = 1; n2 < patterns.size(); n2++) //starting with the 2nd element
					{
						if(n2 >= m_k) //reached m_k
							break;
						USET sn2(patternsSupports[n2].begin(), patternsSupports[n2].end()); //kiwi2, change to LSET
						for(int n1 = n2 - 1; n1 >= 0; n1 --)
						{
							if(patterns[n2].first == patterns[n1].first) //if same support, they must equal, otherwise, not subset of each other
							//in this case, since the last item of the two patterns are equal, remove either pattern would be ok
							{
								if(patternsSupports[n2] == patternsSupports[n1])
								{
									patternsSupports.erase(patternsSupports.begin() + n2);
									patterns.erase(patterns.begin() + n2);
									n2--; //the new one taking the place need to be checked too
									break;
								}
							}
							
							//else
							//before, a later pattern cannot have more support than an early pattern, now it's possible, so both of them maybe removed according to this criterion.
							//change to the following, and need to add more else
							else if(patternsSupports[n2] < patternsSupports[n1])
							{
								USET sn1(patternsSupports[n1].begin(), patternsSupports[n1].end());//kiwi2, change to LSET
								USET dif;//kiwi2, change to LSET
								set_difference(sn2.begin(),sn2.end(),sn1.begin(),sn1.end(),insert_iterator<USET>(dif,dif.begin()));//kiwi2, change to LSET
								if(dif.empty()) 
								{
									patternsSupports.erase(patternsSupports.begin() + n2);
									patterns.erase(patterns.begin() + n2);
									n2--;
									break;
								}
							}
							//the following is for clustering2, before it's impossible to remove p1, not it's possible
							else // (patternsSupports[n2] > patternsSupports[n1]), trying to remove p1
							{
								USET sn1(patternsSupports[n1].begin(), patternsSupports[n1].end());//kiwi2, change to LSET
								USET dif;//kiwi2, change to LSET
								set_difference(sn1.begin(),sn1.end(),sn2.begin(),sn2.end(),insert_iterator<USET>(dif,dif.begin()));//kiwi2, change to LSET
								
								if(dif.empty()) 
								{
									patternsSupports.erase(patternsSupports.begin() + n1);
									patterns.erase(patterns.begin() + n1);
									//for clustering2, even if p1 is removed, still need to n2--, only after n2--, p2 is the same p2, that need to continue to check
									n2--;
									//break;
								}
							}
						}
					}
				}
				break;
			}

			if(patterns.size() > m_k)
			{
				patterns.resize (m_k);
				patternsSupports.resize (m_k);
			}

			//open source. commented out
			/*else if(realSize >= m_k && m_deficit < m_k - patterns.size()) 
				//realSize > m_k guarantees, m_k - patterns.size() is from the removal of subsets
				//otherwise, say, the last level, realSize = candidates.size() < m_k, then m_deficit can be m_k
				m_deficit = m_k - patterns.size();
				*/
		}
			
		//open source. commented out
		//patternSupport
		/*str.Format(_T("%d"), level);
		output += "Level " + str + ":\r\n";
		int patternSupport = 0;
		for (int y = 0; y < patterns.size(); y++)
			patternSupport += patterns[y].first;
		str.Format(_T("%d"), patternSupport);
		output += "Pattern support = " + str + "\r\n";
		*/

		//now, only patterns and patternsSupport left, they'd be useful for the next level
				
		//insert kpatterns for this level into allPatterns
		if(patterns.size() > 0)
		{
			for (int ee = 0; ee < patterns.size(); ee++)
			{
				LROW s;
				int pointer = patterns[ee].second / columnSize;
				short item = patterns[ee].second % columnSize;
				s.push_back (ee); //index for this pattern
				s.push_back(pointer); //pointer to previous level
				s.push_back(item); //current item
				s.push_back (patterns[ee].first); //support
				kpatterns.push_back(s);
					
				patterns[ee].second = item; //now, in patterns, each row's second column is the item itself
			}
			allPatterns.push_back (kpatterns);
			kpatterns.clear();
		
			//open source. commented out				
			//get running time information for output
			/*levelFinish = clock();
			runTime = (double)(levelFinish - levelStart) / CLOCKS_PER_SEC;
			str = gcvt(runTime, 5, charStr);
			output += "Run time = " + str + "\r\n\r\n";
			*/
		
		}
		
		else
		{
			//open source. commented out
			//get running time information for output
			/*levelFinish = clock();
			runTime = (double)(levelFinish - levelStart) / CLOCKS_PER_SEC;
			str = gcvt(runTime, 5, charStr);
			output += "Run time = " + str + "\r\n\r\n";
			*/
		    break;
		}
		
	}

	finish = clock();
	m_clusteringTime = (double)(finish - start) / CLOCKS_PER_SEC;
	//str = gcvt(m_clusteringTime, 5, charStr);//open source. commented out
	//output += "----------------------------------\r\nTotal Run time in seconds = " + str;//open source. commented out
	//free memory, majorly candidates and lastcandidates
	patterns.clear(); //can I use this to free memory? if not, using loop impossible after .clear(), say, candidates
	patternsSupports.clear();

	//write to file

	//open source, added for *.pat first line
	CString info = "";
	char charStr [20];
	CString str;	
	info += "File Name = " + m_dataSource + "; ";

	//date and time info
	_strdate( charStr);
    str = charStr;
	str = "Date = " + str + "; Time = ";
	info += str;
	_strtime( charStr);
	str =  charStr;
	info += str + "; ";
	
	
	//for parameter settings
	str.Format(_T("%d"), m_noise);
	info += "noise as = " + str + "; ";
	str.Format(_T("%d"), m_seqLengthThreshold);
	info += "length >= " + str + "; ";
	str.Format(_T("%d"), m_numberOfRemoved);
	info += "removed = " + str + "; ";
	
	str.Format(_T("%d"), m_k);
	info += "k = " + str + "; ";
	str.Format(_T("%d"), m_w);
	info += "w = " + str + "; ";
	str.Format(_T("%d"), m_min_row);
	info += "min_row = " + str + "; ";

	str.Format(_T("%d"), m_subset);
	info += "remove redundancy = " + str + "\r\n";
	
	CString fileName = m_dataSource;
	fileName.Replace(".txt", "");
	
	//if m_timestampInFileName, prepare date and time info to write
	if(m_timestampInFileName)
	{
		_strdate(charStr);
		str = charStr;
		_strtime( charStr);
		str +=  charStr;
		str.Remove('/');
		str.Remove(':');
	}
	else
		str = "";

	//writeFile(m_writeToDir + fileName + str + ".tim", output);//open source. commented out
	
	//output = "";//open source. commented out

	//write allPatterns to file;
	//writeFile(m_writeToDir + fileName + str + ".pat", toStringAllPatterns());

	if(allPatterns.empty())
		return;

	m_patternSource = fileName + str + ".pat";

	CString totalName = m_writeToDir + fileName + str + ".pat";
	ofstream myfile (totalName);
  
	if (myfile.is_open())
	{
		//open source, write the basic info
		LPCTSTR line = info;
		myfile << line;
		//produce the first line
		CString str, firstLine;
		firstLine = "-level 0-";
		for (int p = 1; p < allPatterns.size(); p++)
		{
			str.Format(_T("%d"), p);
			str = "\t-level " + str + "-"; 
			firstLine += str;
		}
		firstLine += "\n";
		line = firstLine;
		myfile << line;

		//produce the String for all the patterns
		CString str0, str1, str2, str3, lineStr;
		for (int i = 0; i < m_k; i++)
		{
			lineStr = "";
			for(int j = 0; j < allPatterns.size(); j++)
			{
				if(i < allPatterns[j].size())
				{
					str0.Format(_T("%d"), allPatterns[j][i][0]);
					str1.Format(_T("%d"), allPatterns[j][i][1]);
					str2.Format(_T("%d"), allPatterns[j][i][2]);
					str3.Format(_T("%d"), allPatterns[j][i][3]);
					lineStr += str0 + ";" + str1 + ";" + str2 + ";" + str3 + "\t";
				}
				else //need here to take the space for that level
				{
					str0 = "N";
					lineStr += str0 + "\t";
				}
			}
			lineStr.SetAt(lineStr.GetLength() - 1, '\n');
			LPCTSTR line = lineStr;
			myfile << line;
		}
		myfile.close();
	}
	else 
		m_show = "Unable to open file";

	////////////////////
	//update the query zone
	m_queryTopK = 0;
	for (int q = 1; q < allPatterns.size(); q++) //q = 0 is meaningless
		m_queryTopK += allPatterns[q].size();
	
	m_queryLevelHigh = allPatterns.size();
	m_queryRowIndexHigh = m_k;
	UpdateData(false);
}

void CKiwiDlg::OnShowAllPatterns() 
{
	m_show = toStringAllPatterns();
	UpdateData(false);	
}

void CKiwiDlg::Onabout() 
{
	m_show = "KiWi 1.0\tFeburary, 2006\r\nContact: Byron Gao\tbgao@cs.sfu.ca\r\n\r\nReferences:\r\n[1] Byron J. Gao, Obi L. Griffith, Martin Ester, and J.M. Jones. Discovering Significant OPSM Subspace Clusters in Massive Gene Expression Data. \r\n[2] Obi L. Griffith, Byron J. Gao, Mikhail Bilenky, Yuliya Prychyna, Martin Ester, and J.M. Jones. KiWi: A Novel Subspace Clustering Algorithm for the Identification of Coregulated Genes from Massive Gene Expression Datasets.";
	UpdateData(false);

	
}
